import numpy as np

from ..feature.util import (
    FeatureDetector,
    DescriptorExtractor,
    _mask_border_keypoints,
    _prepare_grayscale_input_2D,
)

from ..feature import corner_fast, corner_orientations, corner_peaks, corner_harris
from ..transform import pyramid_gaussian
from .._shared.utils import check_nD

OFAST_MASK = ...
OFAST_UMAX: list = ...

class ORB(FeatureDetector, DescriptorExtractor):
    def __init__(
        self,
        downscale: float = 1.2,
        n_scales: int = 8,
        n_keypoints: int = 500,
        fast_n: int = 9,
        fast_threshold: float = 0.08,
        harris_k: float = 0.04,
    ): ...
    def _build_pyramid(self, image): ...
    def _detect_octave(self, octave_image): ...
    def detect(self, image): ...
    def _extract_octave(self, octave_image, keypoints, orientations): ...
    def extract(self, image, keypoints, scales, orientations): ...
    def detect_and_extract(self, image): ...



from ..classes.multigraph import MultiGraph
from ..classes.reportviews import (
    OutMultiEdgeView,
    InMultiEdgeView,
    DiMultiDegreeView,
)
from copy import deepcopy
from functools import cached_property

from ..classes.graph import Graph
import networkx.convert as convert
from ..classes.coreviews import MultiAdjacencyView
from ..classes.digraph import DiGraph
from ..classes.multigraph import MultiGraph
from ..classes.reportviews import (
    DiMultiDegreeView,
    InMultiDegreeView,
    InMultiEdgeView,
    OutMultiDegreeView,
    OutMultiEdgeView,
)
from ..exception import NetworkXError

__all__ = ["MultiDiGraph"]

class MultiDiGraph(MultiGraph, DiGraph):

    # node_dict_factory = dict    # already assigned in Graph
    # adjlist_outer_dict_factory = dict
    # adjlist_inner_dict_factory = dict
    edge_key_dict_factory = ...
    # edge_attr_dict_factory = dict

    def __init__(self, incoming_graph_data=None, multigraph_input=None, **attr): ...
    @cached_property
    def adj(self): ...
    @cached_property
    def succ(self): ...
    @cached_property
    def pred(self): ...
    def add_edge(self, u_for_edge, v_for_edge, key=None, **attr): ...
    def remove_edge(self, u, v, key=None): ...
    @cached_property
    def edges(self) -> OutMultiEdgeView: ...

    # alias out_edges to edges
    @cached_property
    def out_edges(self) -> OutMultiEdgeView: ...

    out_edges.__doc__ = ...

    @cached_property
    def in_edges(self) -> InMultiEdgeView: ...
    @cached_property
    def degree(self) -> DiMultiDegreeView | int: ...
    @cached_property
    def in_degree(self): ...
    @cached_property
    def out_degree(self): ...
    def is_multigraph(self): ...
    def is_directed(self): ...
    def to_undirected(self, reciprocal=False, as_view=False) -> MultiGraph: ...
    def reverse(self, copy=True): ...



from numpy.typing import NDArray
from typing import Literal
import numpy as np

from .._shared.utils import deprecate_kwarg

@deprecate_kwarg(
    kwarg_mapping={"selem": "footprint"},
    removed_version="1.0",
    deprecated_version="0.19",
)
def reconstruction(
    seed: NDArray,
    mask: NDArray,
    method: Literal["erosion", "dilation"] = "dilation",
    footprint: NDArray | None = None,
    offset: NDArray | None = None,
) -> NDArray: ...



import numpy as np
from typing import Any, Iterable
from .units import AxisInfo
from .axis import Axis

from . import ticker, units

class StrCategoryConverter(units.ConversionInterface):
    @staticmethod
    def convert(
        value: str | Iterable, unit: UnitData, axis: Axis
    ) -> float | np.ndarray: ...
    @staticmethod
    def axisinfo(unit: UnitData, axis: Axis) -> AxisInfo: ...
    @staticmethod
    def default_units(data: str, axis: Axis) -> UnitData: ...

class StrCategoryLocator(ticker.Locator):
    def __init__(self, units_mapping: dict[str, int]) -> None: ...
    def __call__(self): ...
    def tick_values(self, vmin, vmax) -> list: ...

class StrCategoryFormatter(ticker.Formatter):
    def __init__(self, units_mapping: dict[str, int]) -> None: ...
    def __call__(self, x, pos=...): ...
    def format_ticks(self, values)-> list[str]: ...

class UnitData:
    def __init__(self, data=...) -> None: ...
    def update(self, data: bytes)-> None: ...



from matplotlib.backend_bases import (
    FigureCanvasBase,
    FigureManagerBase,
    GraphicsContextBase,
    RendererBase,
    _Backend,
)
from matplotlib._enums import CapStyle, JoinStyle

backend_version = ...


class RendererCairo(RendererBase):
    def __init__(self, dpi) -> None: ...
    def set_context(self, ctx)-> None: ...
    def set_ctx_from_surface(self, surface)-> None: ...
    def set_width_height(self, width, height)-> None: ...
    def draw_path(self, gc, path, transform, rgbFace=...)-> None: ...
    def draw_markers(
        self, gc, marker_path, marker_trans, path, transform, rgbFace=...
    )-> None: ...
    def draw_image(self, gc, x, y, im)-> None: ...
    def draw_text(self, gc, x, y, s, prop, angle, ismath=..., mtext=...)-> None: ...
    def get_canvas_width_height(self)-> tuple[float, float]: ...
    def get_text_width_height_descent(self, s, prop, ismath)-> tuple[float, float, float]: ...
    def new_gc(self)-> GraphicsContextCairo: ...
    def points_to_pixels(self, points)-> float: ...

class GraphicsContextCairo(GraphicsContextBase):
    def __init__(self, renderer) -> None: ...
    def restore(self)-> None: ...
    def set_alpha(self, alpha: float)-> None: ...
    def set_antialiased(self, b: bool) -> None: ...
    def set_capstyle(self, cs: CapStyle) -> None: ...
    def set_clip_rectangle(self, rectangle) -> None: ...
    def set_clip_path(self, path) -> None: ...
    def set_dashes(self, offset, dashes)-> None: ...
    def set_foreground(self, fg, isRGBA: bool|None=None) -> None: ...
    def get_rgb(self) -> tuple[float, float, float]: ...
    def set_joinstyle(self, js: JoinStyle) -> None: ...
    def set_linewidth(self, w: float)-> None: ...

class _CairoRegion:
    def __init__(self, slices, data) -> None: ...

class FigureCanvasCairo(FigureCanvasBase):
    def copy_from_bbox(self, bbox) -> _CairoRegion: ...
    def restore_region(self, region) -> None: ...
    def print_png(self, fobj)-> None: ...
    def print_rgba(self, fobj) -> None: ...

    print_raw = print_rgba
    def print_pdf(self, fobj, *, orientation='portrait') -> None: ...
    def print_ps(self, fobj, *, orientation='portrait') -> None: ...
    def print_svg(self, fobj, *, orientation='portrait') -> None: ...
    def print_svgz(self, fobj, *, orientation='portrait') -> None: ...

class _BackendCairo(_Backend):
    FigureCanvas = FigureCanvasCairo
    FigureManager = FigureManagerBase



from typing import ClassVar, Literal, TypeVar
from abc import ABCMeta, abstractmethod
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from numpy import ndarray
from ..utils.extmath import svd_flip as svd_flip
from scipy.linalg import svd, pinv as pinv2, pinv2 as pinv2
from numbers import Integral as Integral, Real as Real
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ..base import (
    BaseEstimator,
    RegressorMixin,
    TransformerMixin,
    MultiOutputMixin,
    ClassNamePrefixFeaturesOutMixin,
)
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..utils import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
)
from ..utils.fixes import sp_version as sp_version, parse_version as parse_version
from ..utils.validation import (
    check_is_fitted as check_is_fitted,
    FLOAT_DTYPES as FLOAT_DTYPES,
)

PLSSVD_Self = TypeVar("PLSSVD_Self", bound="PLSSVD")
PLSRegression_Self = TypeVar("PLSRegression_Self", bound="PLSRegression")
_PLS_Self = TypeVar("_PLS_Self", bound="_PLS")


import warnings

import numpy as np

__all__ = ["PLSCanonical", "PLSRegression", "PLSSVD"]


class _PLS(
    ClassNamePrefixFeaturesOutMixin,
    TransformerMixin,
    RegressorMixin,
    MultiOutputMixin,
    BaseEstimator,
    metaclass=ABCMeta,
):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        n_components: int = 2,
        *,
        scale: bool = True,
        deflation_mode: str = "regression",
        mode: str = "A",
        algorithm: str = "nipals",
        max_iter: int = 500,
        tol: float = 1e-06,
        copy: bool = True,
    ) -> None:
        ...

    def fit(self: _PLS_Self, X: MatrixLike, Y: MatrixLike | ArrayLike) -> _PLS_Self:
        ...

    def transform(
        self, X: MatrixLike, Y: None | MatrixLike = None, copy: bool = True
    ) -> tuple[ndarray, ...] | ndarray | tuple[ndarray, ndarray]:
        ...

    def inverse_transform(
        self, X: MatrixLike, Y: None | MatrixLike = None
    ) -> tuple[ndarray, ndarray]:
        ...

    def predict(self, X: MatrixLike, copy: bool = True) -> ndarray:
        ...

    def fit_transform(self, X: MatrixLike, y: None | MatrixLike = None) -> ndarray:
        ...

    def coef_(self) -> ndarray:
        ...


class PLSRegression(_PLS):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: list = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...
    y_rotations_: ndarray = ...
    x_rotations_: ndarray = ...
    y_scores_: ndarray = ...
    x_scores_: ndarray = ...
    y_loadings_: ndarray = ...
    x_loadings_: ndarray = ...
    y_weights_: ndarray = ...
    x_weights_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...
    for param in ("deflation_mode", "mode", "algorithm"):
        pass

    # This implementation provides the same results that 3 PLS packages
    # provided in the R language (R-project):
    #     - "mixOmics" with function pls(X, Y, mode = "regression")
    #     - "plspm " with function plsreg2(X, Y)
    #     - "pls" with function oscorespls.fit(X, Y)

    def __init__(
        self,
        n_components: Int = 2,
        *,
        scale: bool = True,
        max_iter: Int = 500,
        tol: Float = 1e-06,
        copy: bool = True,
    ) -> None:
        ...

    def fit(
        self: PLSRegression_Self, X: MatrixLike, Y: MatrixLike | ArrayLike
    ) -> PLSRegression_Self:
        ...


class PLSCanonical(_PLS):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: list = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...
    y_rotations_: ndarray = ...
    x_rotations_: ndarray = ...
    y_loadings_: ndarray = ...
    x_loadings_: ndarray = ...
    y_weights_: ndarray = ...
    x_weights_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...
    for param in ("deflation_mode", "mode"):
        pass

    # This implementation provides the same results that the "plspm" package
    # provided in the R language (R-project), using the function plsca(X, Y).
    # Results are equal or collinear with the function
    # ``pls(..., mode = "canonical")`` of the "mixOmics" package. The
    # difference relies in the fact that mixOmics implementation does not
    # exactly implement the Wold algorithm since it does not normalize
    # y_weights to one.

    def __init__(
        self,
        n_components: Int = 2,
        *,
        scale: bool = True,
        algorithm: Literal["nipals", "svd", "nipals"] = "nipals",
        max_iter: Int = 500,
        tol: Float = 1e-06,
        copy: bool = True,
    ) -> None:
        ...


class CCA(_PLS):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: list = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...
    y_rotations_: ndarray = ...
    x_rotations_: ndarray = ...
    y_loadings_: ndarray = ...
    x_loadings_: ndarray = ...
    y_weights_: ndarray = ...
    x_weights_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...
    for param in ("deflation_mode", "mode", "algorithm"):
        pass

    def __init__(
        self,
        n_components: Int = 2,
        *,
        scale: bool = True,
        max_iter: Int = 500,
        tol: Float = 1e-06,
        copy: bool = True,
    ) -> None:
        ...


class PLSSVD(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    y_weights_: ndarray = ...
    x_weights_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self, n_components: Int = 2, *, scale: bool = True, copy: bool = True
    ) -> None:
        ...

    def fit(self: PLSSVD_Self, X: MatrixLike, Y: MatrixLike | ArrayLike) -> PLSSVD_Self:
        ...

    def transform(
        self, X: MatrixLike, Y: None | MatrixLike | ArrayLike = None
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...

    def fit_transform(
        self, X: MatrixLike, y: None | MatrixLike | ArrayLike = None
    ) -> ndarray | tuple[ndarray, ndarray]:
        ...



import threading
import unittest
from typing import (
    Any,
    Callable,
    ClassVar,
    Dict,
    Iterable,
    List,
    Optional,
    Set,
    Tuple,
    Type,
    Union,
    overload,
)

from django.core.exceptions import ImproperlyConfigured
from django.core.handlers.wsgi import WSGIHandler
from django.core.servers.basehttp import ThreadedWSGIServer, WSGIRequestHandler
from django.db import connections as connections  # noqa: F401
from django.db.backends.sqlite3.base import DatabaseWrapper
from django.db.models.query import _BaseQuerySet
from django.forms.fields import EmailField
from django.http.response import HttpResponse, HttpResponseBase
from django.template.base import Template
from django.test.client import Client
from django.test.utils import CaptureQueriesContext, ContextList
from django.utils.safestring import SafeText

class _AssertNumQueriesContext(CaptureQueriesContext):
    test_case: SimpleTestCase = ...
    num: int = ...
    def __init__(self, test_case: Any, num: Any, connection: Any) -> None: ...

class _AssertTemplateUsedContext:
    test_case: SimpleTestCase = ...
    template_name: str = ...
    rendered_templates: List[Template] = ...
    rendered_template_names: List[str] = ...
    context: ContextList = ...
    def __init__(self, test_case: Any, template_name: Any) -> None: ...
    def on_template_render(
        self, sender: Any, signal: Any, template: Any, context: Any, **kwargs: Any
    ) -> None: ...
    def test(self) -> Any: ...
    def message(self) -> Any: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, exc_type: Any, exc_value: Any, traceback: Any) -> Any: ...

class _AssertTemplateNotUsedContext(_AssertTemplateUsedContext): ...

class _CursorFailure:
    cls_name: str = ...
    wrapped: Callable[..., Any] = ...
    def __init__(self, cls_name: Any, wrapped: Any) -> None: ...
    def __call__(self) -> None: ...

class SimpleTestCase(unittest.TestCase):
    client_class: Any = ...
    client: Client
    allow_database_queries: bool = ...
    # TODO: str -> Literal['__all__']
    databases: Union[Set[str], str] = ...
    def __call__(self, result: Optional[unittest.TestResult] = ...) -> None: ...
    def settings(self, **kwargs: Any) -> Any: ...
    def modify_settings(self, **kwargs: Any) -> Any: ...
    def assertRedirects(
        self,
        response: HttpResponse,
        expected_url: str,
        status_code: int = ...,
        target_status_code: int = ...,
        msg_prefix: str = ...,
        fetch_redirect_response: bool = ...,
    ) -> None: ...
    def assertContains(
        self,
        response: HttpResponseBase,
        text: Union[bytes, int, str],
        count: Optional[int] = ...,
        status_code: int = ...,
        msg_prefix: str = ...,
        html: bool = ...,
    ) -> None: ...
    def assertNotContains(
        self,
        response: HttpResponse,
        text: Union[bytes, str],
        status_code: int = ...,
        msg_prefix: str = ...,
        html: bool = ...,
    ) -> None: ...
    def assertFormError(
        self,
        response: HttpResponse,
        form: str,
        field: Optional[str],
        errors: Union[List[str], str],
        msg_prefix: str = ...,
    ) -> None: ...
    def assertFormsetError(
        self,
        response: HttpResponse,
        formset: str,
        form_index: Optional[int],
        field: Optional[str],
        errors: Union[List[str], str],
        msg_prefix: str = ...,
    ) -> None: ...
    def assertTemplateUsed(
        self,
        response: Optional[Union[HttpResponse, str]] = ...,
        template_name: Optional[str] = ...,
        msg_prefix: str = ...,
        count: Optional[int] = ...,
    ) -> Optional[_AssertTemplateUsedContext]: ...
    def assertTemplateNotUsed(
        self,
        response: Union[HttpResponse, str] = ...,
        template_name: Optional[str] = ...,
        msg_prefix: str = ...,
    ) -> Optional[_AssertTemplateNotUsedContext]: ...
    def assertRaisesMessage(
        self,
        expected_exception: Type[Exception],
        expected_message: str,
        *args: Any,
        **kwargs: Any
    ) -> Any: ...
    def assertWarnsMessage(
        self,
        expected_warning: Type[Exception],
        expected_message: str,
        *args: Any,
        **kwargs: Any
    ) -> Any: ...
    def assertFieldOutput(
        self,
        fieldclass: Type[EmailField],
        valid: Dict[str, str],
        invalid: Dict[str, List[str]],
        field_args: None = ...,
        field_kwargs: None = ...,
        empty_value: str = ...,
    ) -> Any: ...
    def assertHTMLEqual(
        self, html1: str, html2: str, msg: Optional[str] = ...
    ) -> None: ...
    def assertHTMLNotEqual(
        self, html1: str, html2: str, msg: Optional[str] = ...
    ) -> None: ...
    def assertInHTML(
        self,
        needle: str,
        haystack: SafeText,
        count: Optional[int] = ...,
        msg_prefix: str = ...,
    ) -> None: ...
    def assertJSONEqual(
        self,
        raw: str,
        expected_data: Union[Dict[str, Any], List[Any], str, int, float, bool, None],
        msg: Optional[str] = ...,
    ) -> None: ...
    def assertJSONNotEqual(
        self,
        raw: str,
        expected_data: Union[Dict[str, Any], List[Any], str, int, float, bool, None],
        msg: Optional[str] = ...,
    ) -> None: ...
    def assertXMLEqual(
        self, xml1: str, xml2: str, msg: Optional[str] = ...
    ) -> None: ...
    def assertXMLNotEqual(
        self, xml1: str, xml2: str, msg: Optional[str] = ...
    ) -> None: ...

class TransactionTestCase(SimpleTestCase):
    reset_sequences: bool = ...
    available_apps: Any = ...
    fixtures: Any = ...
    multi_db: bool = ...
    serialized_rollback: bool = ...
    def assertQuerysetEqual(
        self,
        qs: _BaseQuerySet[Any],
        values: Iterable[Any],
        transform: Union[Callable[..., Any], Type[str]] = ...,
        ordered: bool = ...,
        msg: Optional[str] = ...,
    ) -> None: ...
    @overload
    def assertNumQueries(
        self,
        num: int,
        func: Callable[..., Any],
        *args: Any,
        using: str = ...,
        **kwargs: Any
    ) -> None: ...
    @overload
    def assertNumQueries(
        self, num: int, func: None = ..., *args: Any, using: str = ..., **kwargs: Any
    ) -> _AssertNumQueriesContext: ...

class TestCase(TransactionTestCase):
    @classmethod
    def setUpTestData(cls) -> None: ...

class CheckCondition:
    conditions: Tuple[Tuple[Callable[..., Any], str]] = ...
    def __init__(self, *conditions: Any) -> None: ...
    def add_condition(
        self, condition: Callable[..., Any], reason: str
    ) -> CheckCondition: ...
    def __get__(self, instance: None, cls: Type[TransactionTestCase] = ...) -> bool: ...

def skipIfDBFeature(*features: Any) -> Callable[..., Any]: ...
def skipUnlessDBFeature(*features: Any) -> Callable[..., Any]: ...
def skipUnlessAnyDBFeature(*features: Any) -> Callable[..., Any]: ...

class QuietWSGIRequestHandler(WSGIRequestHandler): ...

class FSFilesHandler(WSGIHandler):
    application: Any = ...
    base_url: Any = ...
    def __init__(self, application: Any) -> None: ...
    def file_path(self, url: Any) -> Any: ...
    def serve(self, request: Any) -> Any: ...

class _StaticFilesHandler(FSFilesHandler):
    def get_base_dir(self) -> Any: ...
    def get_base_url(self) -> Any: ...

class _MediaFilesHandler(FSFilesHandler):
    def get_base_dir(self) -> Any: ...
    def get_base_url(self) -> Any: ...

class LiveServerThread(threading.Thread):
    host: str = ...
    port: int = ...
    is_ready: threading.Event = ...
    error: Optional[ImproperlyConfigured] = ...
    static_handler: Type[WSGIHandler] = ...
    connections_override: Dict[str, Any] = ...
    def __init__(
        self,
        host: str,
        static_handler: Type[WSGIHandler],
        connections_override: Dict[str, DatabaseWrapper] = ...,
        port: int = ...,
    ) -> None: ...
    httpd: ThreadedWSGIServer = ...
    def terminate(self) -> None: ...

class LiveServerTestCase(TransactionTestCase):
    live_server_url: ClassVar[str]
    host: str = ...
    port: int = ...
    server_thread_class: Type[Any] = ...
    server_thread: Any
    static_handler: Any = ...

class SerializeMixin:
    lockfile: Any = ...
    @classmethod
    def setUpClass(cls) -> None: ...
    @classmethod
    def tearDownClass(cls) -> None: ...

def connections_support_transactions() -> bool: ...



from ._unsupervised import NearestNeighbors as NearestNeighbors
from ._graph import (
    kneighbors_graph as kneighbors_graph,
    radius_neighbors_graph as radius_neighbors_graph,
    KNeighborsTransformer as KNeighborsTransformer,
    RadiusNeighborsTransformer as RadiusNeighborsTransformer,
)
from ._nearest_centroid import NearestCentroid as NearestCentroid
from ._kd_tree import KDTree as KDTree
from ._lof import LocalOutlierFactor as LocalOutlierFactor
from ._base import (
    sort_graph_by_row_values as sort_graph_by_row_values,
    VALID_METRICS as VALID_METRICS,
    VALID_METRICS_SPARSE as VALID_METRICS_SPARSE,
)
from ._classification import (
    KNeighborsClassifier as KNeighborsClassifier,
    RadiusNeighborsClassifier as RadiusNeighborsClassifier,
)
from ._regression import (
    KNeighborsRegressor as KNeighborsRegressor,
    RadiusNeighborsRegressor as RadiusNeighborsRegressor,
)
from ._distance_metric import DistanceMetric as DistanceMetric
from ._kde import KernelDensity as KernelDensity
from ._nca import NeighborhoodComponentsAnalysis as NeighborhoodComponentsAnalysis
from ._ball_tree import BallTree as BallTree

__all__ = [
    "BallTree",
    "DistanceMetric",
    "KDTree",
    "KNeighborsClassifier",
    "KNeighborsRegressor",
    "KNeighborsTransformer",
    "NearestCentroid",
    "NearestNeighbors",
    "RadiusNeighborsClassifier",
    "RadiusNeighborsRegressor",
    "RadiusNeighborsTransformer",
    "kneighbors_graph",
    "radius_neighbors_graph",
    "KernelDensity",
    "LocalOutlierFactor",
    "NeighborhoodComponentsAnalysis",
    "sort_graph_by_row_values",
    "VALID_METRICS",
    "VALID_METRICS_SPARSE",
]



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.interpolate.interpnd, version: unspecified
import typing
import builtins as _mod_builtins

class CloughTocher2DInterpolator(NDInterpolatorBase):
    '\n    CloughTocher2DInterpolator(points, values, tol=1e-6)\n\n    Piecewise cubic, C1 smooth, curvature-minimizing interpolant in 2D.\n\n    .. versionadded:: 0.9\n\n    Methods\n    -------\n    __call__\n\n    Parameters\n    ----------\n    points : ndarray of floats, shape (npoints, ndims); or Delaunay\n        Data point coordinates, or a precomputed Delaunay triangulation.\n    values : ndarray of float or complex, shape (npoints, ...)\n        Data values.\n    fill_value : float, optional\n        Value used to fill in for requested points outside of the\n        convex hull of the input points.  If not provided, then\n        the default is ``nan``.\n    tol : float, optional\n        Absolute/relative tolerance for gradient estimation.\n    maxiter : int, optional\n        Maximum number of iterations in gradient estimation.\n    rescale : bool, optional\n        Rescale points to unit cube before performing interpolation.\n        This is useful if some of the input dimensions have\n        incommensurable units and differ by many orders of magnitude.\n\n    Notes\n    -----\n    The interpolant is constructed by triangulating the input data\n    with Qhull [1]_, and constructing a piecewise cubic\n    interpolating Bezier polynomial on each triangle, using a\n    Clough-Tocher scheme [CT]_.  The interpolant is guaranteed to be\n    continuously differentiable.\n\n    The gradients of the interpolant are chosen so that the curvature\n    of the interpolating surface is approximatively minimized. The\n    gradients necessary for this are estimated using the global\n    algorithm described in [Nielson83]_ and [Renka84]_.\n\n    Examples\n    --------\n    We can interpolate values on a 2D plane:\n\n    >>> from scipy.interpolate import CloughTocher2DInterpolator\n    >>> import matplotlib.pyplot as plt\n    >>> np.random.seed(0)\n    >>> x = np.random.random(10) - 0.5\n    >>> y = np.random.random(10) - 0.5\n    >>> z = np.hypot(x, y)\n    >>> X = np.linspace(min(x), max(x))\n    >>> Y = np.linspace(min(y), max(y))\n    >>> X, Y = np.meshgrid(X, Y)  # 2D grid for interpolation\n    >>> interp = CloughTocher2DInterpolator(list(zip(x, y)), z)\n    >>> Z = interp(X, Y)\n    >>> plt.pcolormesh(X, Y, Z, shading=\'auto\')\n    >>> plt.plot(x, y, "ok", label="input point")\n    >>> plt.legend()\n    >>> plt.colorbar()\n    >>> plt.axis("equal")\n    >>> plt.show()\n\n    See also\n    --------\n    griddata :\n        Interpolate unstructured D-D data.\n    LinearNDInterpolator :\n        Piecewise linear interpolant in N dimensions.\n    NearestNDInterpolator :\n        Nearest-neighbor interpolation in N dimensions.\n\n    References\n    ----------\n    .. [1] http://www.qhull.org/\n\n    .. [CT] See, for example,\n       P. Alfeld,\n       \'\'A trivariate Clough-Tocher scheme for tetrahedral data\'\'.\n       Computer Aided Geometric Design, 1, 169 (1984);\n       G. Farin,\n       \'\'Triangular Bernstein-Bezier patches\'\'.\n       Computer Aided Geometric Design, 3, 83 (1986).\n\n    .. [Nielson83] G. Nielson,\n       \'\'A method for interpolating scattered data based upon a minimum norm\n       network\'\'.\n       Math. Comp., 40, 253 (1983).\n\n    .. [Renka84] R. J. Renka and A. K. Cline.\n       \'\'A Triangle-based C1 interpolation method.\'\',\n       Rocky Mountain J. Math., 14, 223 (1984).\n\n    '
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, points, values, tol=...) -> None:
        '\n    CloughTocher2DInterpolator(points, values, tol=1e-6)\n\n    Piecewise cubic, C1 smooth, curvature-minimizing interpolant in 2D.\n\n    .. versionadded:: 0.9\n\n    Methods\n    -------\n    __call__\n\n    Parameters\n    ----------\n    points : ndarray of floats, shape (npoints, ndims); or Delaunay\n        Data point coordinates, or a precomputed Delaunay triangulation.\n    values : ndarray of float or complex, shape (npoints, ...)\n        Data values.\n    fill_value : float, optional\n        Value used to fill in for requested points outside of the\n        convex hull of the input points.  If not provided, then\n        the default is ``nan``.\n    tol : float, optional\n        Absolute/relative tolerance for gradient estimation.\n    maxiter : int, optional\n        Maximum number of iterations in gradient estimation.\n    rescale : bool, optional\n        Rescale points to unit cube before performing interpolation.\n        This is useful if some of the input dimensions have\n        incommensurable units and differ by many orders of magnitude.\n\n    Notes\n    -----\n    The interpolant is constructed by triangulating the input data\n    with Qhull [1]_, and constructing a piecewise cubic\n    interpolating Bezier polynomial on each triangle, using a\n    Clough-Tocher scheme [CT]_.  The interpolant is guaranteed to be\n    continuously differentiable.\n\n    The gradients of the interpolant are chosen so that the curvature\n    of the interpolating surface is approximatively minimized. The\n    gradients necessary for this are estimated using the global\n    algorithm described in [Nielson83]_ and [Renka84]_.\n\n    Examples\n    --------\n    We can interpolate values on a 2D plane:\n\n    >>> from scipy.interpolate import CloughTocher2DInterpolator\n    >>> import matplotlib.pyplot as plt\n    >>> np.random.seed(0)\n    >>> x = np.random.random(10) - 0.5\n    >>> y = np.random.random(10) - 0.5\n    >>> z = np.hypot(x, y)\n    >>> X = np.linspace(min(x), max(x))\n    >>> Y = np.linspace(min(y), max(y))\n    >>> X, Y = np.meshgrid(X, Y)  # 2D grid for interpolation\n    >>> interp = CloughTocher2DInterpolator(list(zip(x, y)), z)\n    >>> Z = interp(X, Y)\n    >>> plt.pcolormesh(X, Y, Z, shading=\'auto\')\n    >>> plt.plot(x, y, "ok", label="input point")\n    >>> plt.legend()\n    >>> plt.colorbar()\n    >>> plt.axis("equal")\n    >>> plt.show()\n\n    See also\n    --------\n    griddata :\n        Interpolate unstructured D-D data.\n    LinearNDInterpolator :\n        Piecewise linear interpolant in N dimensions.\n    NearestNDInterpolator :\n        Nearest-neighbor interpolation in N dimensions.\n\n    References\n    ----------\n    .. [1] http://www.qhull.org/\n\n    .. [CT] See, for example,\n       P. Alfeld,\n       \'\'A trivariate Clough-Tocher scheme for tetrahedral data\'\'.\n       Computer Aided Geometric Design, 1, 169 (1984);\n       G. Farin,\n       \'\'Triangular Bernstein-Bezier patches\'\'.\n       Computer Aided Geometric Design, 3, 83 (1986).\n\n    .. [Nielson83] G. Nielson,\n       \'\'A method for interpolating scattered data based upon a minimum norm\n       network\'\'.\n       Math. Comp., 40, 253 (1983).\n\n    .. [Renka84] R. J. Renka and A. K. Cline.\n       \'\'A Triangle-based C1 interpolation method.\'\',\n       Rocky Mountain J. Math., 14, 223 (1984).\n\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def _do_evaluate(self, xi, dummy) -> typing.Any:
        ...
    
    def _evaluate_complex(self, xi) -> typing.Any:
        ...
    
    def _evaluate_double(self, xi) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class GradientEstimationWarning(_mod_builtins.Warning):
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, *args, **kwargs) -> None:
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __module__: str
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def __weakref__(self) -> typing.Any:
        'list of weak references to the object (if defined)'
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class LinearNDInterpolator(NDInterpolatorBase):
    '\n    LinearNDInterpolator(points, values, fill_value=np.nan, rescale=False)\n\n    Piecewise linear interpolant in N dimensions.\n\n    .. versionadded:: 0.9\n\n    Methods\n    -------\n    __call__\n\n    Parameters\n    ----------\n    points : ndarray of floats, shape (npoints, ndims); or Delaunay\n        Data point coordinates, or a precomputed Delaunay triangulation.\n    values : ndarray of float or complex, shape (npoints, ...)\n        Data values.\n    fill_value : float, optional\n        Value used to fill in for requested points outside of the\n        convex hull of the input points.  If not provided, then\n        the default is ``nan``.\n    rescale : bool, optional\n        Rescale points to unit cube before performing interpolation.\n        This is useful if some of the input dimensions have\n        incommensurable units and differ by many orders of magnitude.\n\n    Notes\n    -----\n    The interpolant is constructed by triangulating the input data\n    with Qhull [1]_, and on each triangle performing linear\n    barycentric interpolation.\n\n    Examples\n    --------\n    We can interpolate values on a 2D plane:\n\n    >>> from scipy.interpolate import LinearNDInterpolator\n    >>> import matplotlib.pyplot as plt\n    >>> np.random.seed(0)\n    >>> x = np.random.random(10) - 0.5\n    >>> y = np.random.random(10) - 0.5\n    >>> z = np.hypot(x, y)\n    >>> X = np.linspace(min(x), max(x))\n    >>> Y = np.linspace(min(y), max(y))\n    >>> X, Y = np.meshgrid(X, Y)  # 2D grid for interpolation\n    >>> interp = LinearNDInterpolator(list(zip(x, y)), z)\n    >>> Z = interp(X, Y)\n    >>> plt.pcolormesh(X, Y, Z, shading=\'auto\')\n    >>> plt.plot(x, y, "ok", label="input point")\n    >>> plt.legend()\n    >>> plt.colorbar()\n    >>> plt.axis("equal")\n    >>> plt.show()\n\n    See also\n    --------\n    griddata :\n        Interpolate unstructured D-D data.\n    NearestNDInterpolator :\n        Nearest-neighbor interpolation in N dimensions.\n    CloughTocher2DInterpolator :\n        Piecewise cubic, C1 smooth, curvature-minimizing interpolant in 2D.\n\n    References\n    ----------\n    .. [1] http://www.qhull.org/\n\n    '
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, points, values, fill_value=..., rescale=...) -> None:
        '\n    LinearNDInterpolator(points, values, fill_value=np.nan, rescale=False)\n\n    Piecewise linear interpolant in N dimensions.\n\n    .. versionadded:: 0.9\n\n    Methods\n    -------\n    __call__\n\n    Parameters\n    ----------\n    points : ndarray of floats, shape (npoints, ndims); or Delaunay\n        Data point coordinates, or a precomputed Delaunay triangulation.\n    values : ndarray of float or complex, shape (npoints, ...)\n        Data values.\n    fill_value : float, optional\n        Value used to fill in for requested points outside of the\n        convex hull of the input points.  If not provided, then\n        the default is ``nan``.\n    rescale : bool, optional\n        Rescale points to unit cube before performing interpolation.\n        This is useful if some of the input dimensions have\n        incommensurable units and differ by many orders of magnitude.\n\n    Notes\n    -----\n    The interpolant is constructed by triangulating the input data\n    with Qhull [1]_, and on each triangle performing linear\n    barycentric interpolation.\n\n    Examples\n    --------\n    We can interpolate values on a 2D plane:\n\n    >>> from scipy.interpolate import LinearNDInterpolator\n    >>> import matplotlib.pyplot as plt\n    >>> np.random.seed(0)\n    >>> x = np.random.random(10) - 0.5\n    >>> y = np.random.random(10) - 0.5\n    >>> z = np.hypot(x, y)\n    >>> X = np.linspace(min(x), max(x))\n    >>> Y = np.linspace(min(y), max(y))\n    >>> X, Y = np.meshgrid(X, Y)  # 2D grid for interpolation\n    >>> interp = LinearNDInterpolator(list(zip(x, y)), z)\n    >>> Z = interp(X, Y)\n    >>> plt.pcolormesh(X, Y, Z, shading=\'auto\')\n    >>> plt.plot(x, y, "ok", label="input point")\n    >>> plt.legend()\n    >>> plt.colorbar()\n    >>> plt.axis("equal")\n    >>> plt.show()\n\n    See also\n    --------\n    griddata :\n        Interpolate unstructured D-D data.\n    NearestNDInterpolator :\n        Nearest-neighbor interpolation in N dimensions.\n    CloughTocher2DInterpolator :\n        Piecewise cubic, C1 smooth, curvature-minimizing interpolant in 2D.\n\n    References\n    ----------\n    .. [1] http://www.qhull.org/\n\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def _do_evaluate(self, xi, dummy) -> typing.Any:
        ...
    
    def _evaluate_complex(self, xi) -> typing.Any:
        ...
    
    def _evaluate_double(self, xi) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

class NDInterpolatorBase(_mod_builtins.object):
    '\n    Common routines for interpolators.\n\n    .. versionadded:: 0.9\n\n    '
    def __call__(self, *args) -> typing.Any:
        '\n        interpolator(xi)\n\n        Evaluate interpolator at given points.\n\n        Parameters\n        ----------\n        x1, x2, ... xn: array-like of float\n            Points where to interpolate data at.\n            x1, x2, ... xn can be array-like of float with broadcastable shape.\n            or x1 can be array-like of float with shape ``(..., ndim)``\n\n        '
        ...
    
    __dict__: typing.Dict[str, typing.Any]
    def __init__(self, points, values, fill_value, ndim, rescale, need_contiguous, need_values) -> None:
        '\n        Check shape of points and values arrays, and reshape values to\n        (npoints, nvalues).  Ensure the `points` and values arrays are\n        C-contiguous, and of correct type.\n        '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    __module__: str
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    @property
    def __weakref__(self) -> typing.Any:
        'list of weak references to the object (if defined)'
        ...
    
    def _check_call_shape(self, xi) -> typing.Any:
        ...
    
    def _scale_x(self, xi) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: str
__file__: str
__name__: str
__package__: str
def __pyx_unpickle_Enum() -> typing.Any:
    ...

__test__: dict
def _ndim_coords_from_arrays() -> typing.Any:
    '\n    Convert a tuple of coordinate arrays to a (..., ndim)-shaped array.\n\n    '
    ...

def estimate_gradients_2d_global() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




import datetime
from matplotlib.contour import QuadContourSet
import numpy as np
from typing import Callable, ContextManager, Literal, Sequence, overload
from matplotlib import rcParams as rcParams
from matplotlib import style as style
from ._typing import *
from .tri.tricontour import TriContourSet
from .transforms import Bbox, Transform
from .text import Text, Annotation
from .container import BarContainer, ErrorbarContainer, StemContainer
from .patches import Polygon, FancyArrow
from .quiver import Quiver
from .colors import Colormap, Normalize
from .backend_bases import FigureCanvasBase, MouseButton, FigureManagerBase
from .collections import (
    LineCollection,
    Collection,
    QuadMesh,
    PathCollection,
    PolyCollection,
    BrokenBarHCollection,
)
from .lines import Line2D
from .legend import Legend
from .image import AxesImage, FigureImage
from .figure import Figure, SubFigure
from .scale import ScaleBase
from .backend_bases import _Backend
from .axes import Axes as Axes
from .artist import Artist
from .table import Table
from .widgets import SubplotTool
from .markers import MarkerStyle
from .streamplot import StreamplotSet

from array import array
import matplotlib
import matplotlib.image
from . import rcParams
from .rcsetup import interactive_bk as _interactive_bk

def install_repl_displayhook()-> None: ...
def uninstall_repl_displayhook()-> None: ...

def draw_all(force: bool = False)-> None: ...

def set_loglevel(*args, **kwargs)-> None: ...
def findobj(o=..., match=..., include_self: bool = ...) -> list: ...
def switch_backend(newbackend: str)-> None:
    class backend_mod(_Backend): ...

def new_figure_manager(*args, **kwargs)-> FigureManagerBase: ...
def draw_if_interactive(*args, **kwargs): ...
def show(*args, **kwargs): ...
def isinteractive() -> bool: ...

class _IoffContext:
    def __init__(self) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback): ...

class _IonContext:
    def __init__(self) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback): ...

def ioff()-> _IoffContext: ...
def ion()-> _IonContext: ...
def pause(interval)-> None: ...
def rc(group, **kwargs)-> None: ...
def rc_context(rc: dict = ..., fname: str | PathLike = ...)-> ContextManager: ...
def rcdefaults()-> None: ...
def getp(obj: Artist, *args, **kwargs): ...
def get(obj: Artist, *args, **kwargs): ...
def setp(obj: Artist | list, *args, **kwargs): ...
def xkcd(scale: float = ..., length: float = ..., randomness: float = ...)-> _xkcd: ...

class _xkcd:
    def __init__(self, scale: float, length: float, randomness: float) -> None: ...
    def __enter__(self)-> _xkcd: ...
    def __exit__(self, *args)-> None: ...

def figure(
    num: int | str | Figure | SubFigure| None = None,
    figsize: Sequence[float]|None = None,
    dpi: float|None = None,
    facecolor: Color|None = None,
    edgecolor: Color|None = None,
    frameon: bool = True,
    FigureClass = ...,
    clear: bool = False,
    **kwargs
) -> Figure: ...
def gcf() -> Figure: ...
def fignum_exists(num) -> bool: ...
def get_fignums() -> list[int]: ...
def get_figlabels(): ...
def get_current_fig_manager() -> FigureManagerBase: ...
def connect(s: str, func: Callable): ...
def disconnect(cid: int): ...
def close(fig: None | int | str | Figure = ...): ...
def clf() -> None: ...
def draw() -> None: ...
def savefig(*args, **kwargs) -> None: ...
def figlegend(*args, **kwargs) -> Legend: ...
def axes(arg: None | tuple = ..., **kwargs) -> Axes: ...
def delaxes(ax: Axes = ...) -> None: ...
def sca(ax: Axes) -> None: ...
def cla() -> None: ...
def subplot(*args, **kwargs) -> Axes: ...


@overload
def subplots(
    nrows: int = ...,
    ncols: int = ...,
    *,
    squeeze: Literal[False],
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...
@overload
def subplots(
    nrows: Literal[1] = ...,
    ncols: Literal[1] = ...,
    *,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, Axes]: ...

##

@overload
def subplots(
    nrows: Literal[1] = ...,
    ncols: int = ...,
    *,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...
@overload
def subplots(
    nrows: int = ...,
    ncols: Literal[1] = ...,
    *,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...
@overload
def subplots(
    *,
    nrows: int = ...,
    ncols: int = ...,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...
@overload
def subplots(
    *,
    ncols: int = ...,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...
@overload
def subplots(
    nrows: int = ...,
    *,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...

@overload
def subplots(
    *,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: Literal[True] = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, Axes]: ...

@overload
def subplots(
    nrows: int = ...,
    ncols: int = ...,
    *,
    sharex: bool | Literal["none", "all", "row", "col"] = ...,
    sharey: bool | Literal["none", "all", "row", "col"] = ...,
    squeeze: bool = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    **fig_kw
) -> tuple[Figure, np.ndarray]: ...

def subplot_mosaic(
    mosaic: list | str,
    *,
    sharex: bool = ...,
    sharey: bool = ...,
    subplot_kw: dict = ...,
    gridspec_kw: dict = ...,
    empty_sentinel: object = ...,
    **fig_kw
) -> tuple[Figure, dict[Text, Axes]]: ...
def subplot2grid(
    shape: Sequence[int],
    loc: Sequence[int],
    rowspan: int = ...,
    colspan: int = ...,
    fig: Figure = ...,
    **kwargs
) -> Axes: ...
def twinx(ax: Axes = ...) -> Axes: ...
def twiny(ax: Axes = ...) -> Axes: ...
def subplot_tool(targetfig: Figure = ...) -> SubplotTool: ...
def box(on: bool | None = ...): ...
def xlim(*args, **kwargs) -> tuple[float, float]: ...
def ylim(*args, **kwargs) -> tuple[float, float]: ...
def xticks(
    ticks: ArrayLike = ..., labels: ArrayLike = ..., **kwargs
) -> tuple[list, list[Text]]: ...
def yticks(
    ticks: ArrayLike = ..., labels: ArrayLike = ..., **kwargs
) -> tuple[list, list[Text]]: ...
def rgrids(
    radii: Sequence[float] = ...,
    labels: Sequence[str] | None = ...,
    angle: float = ...,
    fmt: str | None = ...,
    **kwargs
) -> tuple[list[Line2D], list[Text]]: ...
def thetagrids(
    angles: Sequence[float] = ...,
    labels: Sequence[str] | None = ...,
    fmt: str | None = ...,
    **kwargs
) -> tuple[list[Line2D], list[Text]]: ...
def get_plot_commands() -> list: ...
def colorbar(mappable=..., cax: Axes = ..., ax: Axes = ..., **kwargs): ...
def clim(vmin: float | None = ..., vmax: float | None = ...): ...
def set_cmap(cmap: Colormap | str): ...
def imread(fname: str | FileLike, format: str = ...) -> np.ndarray: ...
def imsave(fname: str | PathLike | FileLike, arr: ArrayLike, **kwargs): ...
def matshow(
    A: ArrayLike, fignum: None | int | Literal[False] = ..., **kwargs
) -> AxesImage: ...
def polar(*args, **kwargs): ...
def figimage(
    X: ArrayLike,
    xo: float = ...,
    yo: float = ...,
    alpha: None | float = ...,
    norm: Normalize = ...,
    cmap: str | Colormap = ...,
    vmin: float = ...,
    vmax: float = ...,
    origin: Literal["upper", "lower"] = ...,
    resize: bool = ...,
    **kwargs
) -> FigureImage: ...
def figtext(
    x: float, y: float, s: str, fontdict: dict = ..., **kwargs
) -> text.Text: ...
def gca(): ...
def gci(): ...
def ginput(
    n: int = ...,
    timeout: float = ...,
    show_clicks: bool = ...,
    mouse_add: MouseButton | None = ...,
    mouse_pop: MouseButton | None = ...,
    mouse_stop: MouseButton | None = ...,
) -> list[tuple]: ...
def subplots_adjust(
    left: float = ...,
    bottom: float = ...,
    right: float = ...,
    top: float = ...,
    wspace: float = ...,
    hspace: float = ...,
): ...
def suptitle(t: str, **kwargs) -> Text: ...
def tight_layout(
    *,
    pad: float = ...,
    h_pad: float = ...,
    w_pad: float = ...,
    rect: Sequence[float] = ...
): ...
def waitforbuttonpress(timeout=...): ...
def acorr(x: ArrayLike, *, data=..., **kwargs): ...
def angle_spectrum(
    x: Sequence[float],
    Fs: float = ...,
    Fc: float = ...,
    window: Callable | np.ndarray = ...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    *,
    data=...,
    **kwargs
): ...
def annotate(
    text: str,
    xy: Sequence[float],
    xytext: Sequence[float] = ...,
    xycoords: str | Artist | Transform | Callable = ...,
    textcoords: str | Artist | Transform | Callable = ...,
    arrowprops: dict = ...,
    annotation_clip: bool | None = ...,
    **kwargs
) -> Annotation: ...
def arrow(x: float, y: float, dx: float, dy: float, **kwargs) -> FancyArrow: ...
def autoscale(
    enable: bool | None = ...,
    axis: Literal["both", "x", "y"] = ...,
    tight: bool | None = ...,
): ...
def axhline(
    y: float = ..., xmin: float = ..., xmax: float = ..., **kwargs
) -> Line2D: ...
def axhspan(
    ymin: float, ymax: float, xmin: float = ..., xmax: float = ..., **kwargs
) -> Polygon: ...
def axis(*args, emit: bool = ..., **kwargs): ...
def axline(
    xy1: Sequence[float], xy2: Sequence[float] = ..., *, slope: float = ..., **kwargs
) -> Line2D: ...
def axvline(
    x: float = ..., ymin: float = ..., ymax: float = ..., **kwargs
) -> Line2D: ...
def axvspan(
    xmin: float, xmax: float, ymin: float = ..., ymax: float = ..., **kwargs
) -> Polygon: ...
def bar(
    x: float | ArrayLike,
    height: float | ArrayLike,
    width: float | ArrayLike = ...,
    bottom: float | ArrayLike = ...,
    *,
    align: Literal["center", "edge"] = ...,
    data=...,
    **kwargs
) -> BarContainer: ...
def barbs(*args, data=..., **kwargs): ...
def barh(
    y: float | ArrayLike,
    width: float | ArrayLike,
    height: float | ArrayLike = ...,
    left: float | ArrayLike = ...,
    *,
    align: Literal["center", "edge"] = ...,
    **kwargs
) -> BarContainer: ...
def bar_label(
    container: BarContainer,
    labels: ArrayLike = ...,
    *,
    fmt: str = ...,
    label_type: Literal["edge", "center"] = ...,
    padding: float = ...,
    **kwargs
) -> list: ...
def boxplot(
    x: ArrayLike,
    notch: bool = ...,
    sym: str = ...,
    vert: bool = ...,
    whis: float = ...,
    positions: ArrayLike = ...,
    widths: float | ArrayLike = ...,
    patch_artist: bool = ...,
    bootstrap: int = ...,
    usermedians=...,
    conf_intervals: ArrayLike = ...,
    meanline: bool = ...,
    showmeans=...,
    showcaps=...,
    showbox=...,
    showfliers=...,
    boxprops=...,
    labels: Sequence = ...,
    flierprops=...,
    medianprops=...,
    meanprops=...,
    capprops=...,
    whiskerprops=...,
    manage_ticks: bool = ...,
    autorange: bool = ...,
    zorder: float = ...,
    capwidths=...,
    *,
    data=...
) -> dict: ...
def broken_barh(xranges, yrange, *, data=..., **kwargs) -> BrokenBarHCollection: ...
def clabel(CS, levels: ArrayLike = ..., **kwargs): ...
def cohere(
    x,
    y,
    NFFT: int = ...,
    Fs: float = ...,
    Fc: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: float = ...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
    *,
    data=...,
    **kwargs
): ...
def contour(*args, data=..., **kwargs) -> QuadContourSet: ...
def contourf(*args, data=..., **kwargs) -> QuadContourSet: ...
def csd(
    x: Sequence[float],
    y: Sequence[float],
    NFFT: int = ...,
    Fs: float = ...,
    Fc: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: float = ...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
    return_line: bool = ...,
    *,
    data=...,
    **kwargs
): ...
def errorbar(
    x: float | ArrayLike,
    y: float | ArrayLike,
    yerr: float | ArrayLike = ...,
    xerr: float | ArrayLike = ...,
    fmt: str = ...,
    ecolor: Color = ...,
    elinewidth: float = ...,
    capsize: float = ...,
    barsabove: bool = ...,
    lolims: bool = ...,
    uplims: bool = ...,
    xlolims: bool = ...,
    xuplims: bool = ...,
    errorevery: int = ...,
    capthick: float = ...,
    *,
    data=...,
    **kwargs
) -> ErrorbarContainer: ...
def eventplot(
    positions: ArrayLike | Sequence[ArrayLike],
    orientation: Literal["horizontal", "vertical"] = ...,
    lineoffsets: float | ArrayLike = ...,
    linelengths: float | ArrayLike = ...,
    linewidths: float | ArrayLike = ...,
    colors: Color | list[Color] = ...,
    linestyles: str | tuple | list = ...,
    *,
    data=...,
    **kwargs
) -> list: ...
def fill(*args, data=..., **kwargs) -> list[Polygon]: ...
def fill_between(
    x,
    y1: Scalar,
    y2: Scalar = ...,
    where: ArrayLike = ...,
    interpolate: bool = ...,
    step: Literal["pre", "post", "mid"] = ...,
    *,
    data=...,
    **kwargs
) -> PolyCollection: ...
def fill_betweenx(
    y,
    x1: Scalar,
    x2: Scalar = ...,
    where: ArrayLike = ...,
    step: Literal["pre", "post", "mid"] = ...,
    interpolate: bool = ...,
    *,
    data=...,
    **kwargs
) -> PolyCollection: ...
def grid(
    visible: bool | None = ...,
    which: Literal["major", "minor", "both"] = ...,
    axis: Literal["both", "x", "y"] = ...,
    **kwargs
): ...
def hexbin(
    x: ArrayLike,
    y: ArrayLike,
    C: ArrayLike = ...,
    gridsize: int = ...,
    bins: Literal["log"] | int | Sequence = ...,
    xscale: Literal["linear", "log"] = ...,
    yscale: Literal["linear", "log"] = ...,
    extent=...,
    cmap=...,
    norm=...,
    vmin=...,
    vmax=...,
    alpha=...,
    linewidths=...,
    edgecolors=...,
    reduce_C_function=...,
    mincnt: int = ...,
    marginals: bool = ...,
    *,
    data=...,
    **kwargs
) -> PolyCollection: ...
def hist(
    x,
    bins: int | Sequence | str = ...,
    range: tuple | None = ...,
    density: bool = ...,
    weights=...,
    cumulative: bool | Literal[-1] = ...,
    bottom=...,
    histtype: Literal["bar", "barstacked", "step", "stepfilled"] = ...,
    align: Literal["left", "mid", "right"] = ...,
    orientation: Literal["vertical", "horizontal"] = ...,
    rwidth: float | None = ...,
    log: bool = ...,
    color: Color | None = ...,
    label: str | None = ...,
    stacked: bool = ...,
    *,
    data=...,
    **kwargs
): ...
def stairs(
    values: ArrayLike,
    edges: ArrayLike = ...,
    *,
    orientation: Literal["vertical", "horizontal"] = ...,
    baseline: float | ArrayLike | None = ...,
    fill: bool = ...,
    data=...,
    **kwargs
): ...
def hist2d(
    x,
    y,
    bins: None | int | ArrayLike = ...,
    range=...,
    density: bool = ...,
    weights=...,
    cmin: float = ...,
    cmax: float = ...,
    *,
    data=...,
    **kwargs
): ...
def hlines(
    y: float | ArrayLike,
    xmin: float | ArrayLike,
    xmax: float | ArrayLike,
    colors: list[Color] = ...,
    linestyles: Literal["solid", "dashed", "dashdot", "dotted"] = ...,
    label: str = ...,
    *,
    data=...,
    **kwargs
) -> LineCollection: ...
def imshow(
    X: ArrayLike,
    cmap: str | Colormap = ...,
    norm: Normalize = ...,
    aspect: Literal["equal", "auto"] | float = ...,
    interpolation: str = ...,
    alpha: float | ArrayLike = ...,
    vmin: float = ...,
    vmax: float = ...,
    origin: Literal["upper", "lower"] = ...,
    extent: Sequence[float] = ...,
    *,
    interpolation_stage: Literal["data", "rgba"] = ...,
    filternorm: bool = ...,
    filterrad: float = ...,
    resample: bool = ...,
    url: str = ...,
    data=...,
    **kwargs
) -> AxesImage: ...
def legend(*args, **kwargs) -> Legend: ...
def locator_params(
    axis: Literal["both", "x", "y"] = ..., tight: bool | None = ..., **kwargs
): ...
def loglog(*args, **kwargs) -> list: ...
def magnitude_spectrum(
    x: Sequence,
    Fs: float = ...,
    Fc: float = ...,
    window: Callable | np.ndarray = ...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale: Literal["default", "linear", "dB"] = ...,
    *,
    data=...,
    **kwargs
): ...
def margins(*margins, x: float = ..., y: float = ..., tight: bool | None = ...): ...
def minorticks_off(): ...
def minorticks_on(): ...
def pcolor(
    *args,
    shading: Literal["flat", "nearest", "auto"] = ...,
    alpha: float = ...,
    norm: Normalize = ...,
    cmap: str | Colormap = ...,
    vmin: float = ...,
    vmax: float = ...,
    data=...,
    **kwargs
) -> Collection: ...
def pcolormesh(
    *args,
    alpha: float = ...,
    norm: Normalize = ...,
    cmap: str | Colormap = ...,
    vmin: float = ...,
    vmax: float = ...,
    shading: Literal["flat", "nearest", "gouraud", "auto"] = ...,
    antialiased=...,
    data=...,
    **kwargs
) -> QuadMesh: ...
def phase_spectrum(
    x: Sequence,
    Fs: float = ...,
    Fc: float = ...,
    window: Callable | np.ndarray = ...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    *,
    data=...,
    **kwargs
): ...
def pie(
    x,
    explode: ArrayLike = ...,
    labels: Sequence[str] = ...,
    colors: ArrayLike = ...,
    autopct: None | str | Callable = ...,
    pctdistance: float = ...,
    shadow: bool = ...,
    labeldistance: float | None = ...,
    startangle: float = ...,
    radius: float = ...,
    counterclock: bool = ...,
    wedgeprops: dict = ...,
    textprops: dict = ...,
    center: tuple[float, float] = ...,
    frame: bool = ...,
    rotatelabels: bool = ...,
    *,
    normalize: bool = ...,
    data=...
): ...
def plot(*args, scalex=..., scaley=..., data=..., **kwargs) -> list: ...
def plot_date(
    x: ArrayLike,
    y: ArrayLike,
    fmt: str = ...,
    tz: datetime.tzinfo = ...,
    xdate: bool = ...,
    ydate: bool = ...,
    *,
    data=...,
    **kwargs
) -> list: ...
def psd(
    x: Sequence,
    NFFT: int = ...,
    Fs: float = ...,
    Fc: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: float = ...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
    return_line: bool = ...,
    *,
    data=...,
    **kwargs
): ...
def quiver(*args, data=..., **kwargs) -> Quiver: ...
def quiverkey(Q: Quiver, X: float, Y: float, U: float, label: str, **kwargs): ...
def scatter(
    x: float | ArrayLike,
    y: float | ArrayLike,
    s: float | ArrayLike = ...,
    c: ArrayLike | list[Color] | Color = ...,
    marker: MarkerStyle = ...,
    cmap: str | Colormap = ...,
    norm: Normalize = ...,
    vmin: float = ...,
    vmax: float = ...,
    alpha: float = ...,
    linewidths: float | ArrayLike = ...,
    *,
    edgecolors: Color = ...,
    plotnonfinite: bool = ...,
    data=...,
    **kwargs
) -> PathCollection: ...
def semilogx(*args, **kwargs) -> list: ...
def semilogy(*args, **kwargs) -> list: ...
def specgram(
    x: Sequence,
    NFFT: int = ...,
    Fs: float = ...,
    Fc: float = ...,
    detrend: Literal["none", "mean", "linear"] | Callable = ...,
    window: Callable | np.ndarray = ...,
    noverlap: int = ...,
    cmap: Colormap = ...,
    xextent=...,
    pad_to: float = ...,
    sides: Literal["default", "onesided", "twosided"] = ...,
    scale_by_freq: bool = ...,
    mode: Literal["default", "psd", "magnitude", "angle", "phase"] = ...,
    scale: Literal["default", "linear", "dB"] = ...,
    vmin=...,
    vmax=...,
    *,
    data=...,
    **kwargs
): ...
def spy(
    Z,
    precision: float | Literal["present"] = ...,
    marker=...,
    markersize=...,
    aspect: Literal["equal", "auto", None] | float = ...,
    origin: Literal["upper", "lower"] = ...,
    **kwargs
) -> tuple[AxesImage, Line2D]: ...
def stackplot(
    x,
    *args,
    labels: list[str] = ...,
    colors: list[Color] = ...,
    baseline: Literal["zero", "sym", "wiggle", "weighted_wiggle"] = ...,
    data=...,
    **kwargs
) -> list: ...
def stem(
    *args,
    linefmt: str = ...,
    markerfmt: str = ...,
    basefmt: str = ...,
    bottom: float = ...,
    label: str = ...,
    use_line_collection: bool = ...,
    orientation: str = ...,
    data=...
) -> StemContainer: ...
def step(
    x: ArrayLike,
    y: ArrayLike,
    *args,
    where: Literal["pre", "post", "mid"] = ...,
    data=...,
    **kwargs
) -> list: ...
def streamplot(
    x,
    y,
    u,
    v,
    density: float = ...,
    linewidth: float = ...,
    color: Color = ...,
    cmap: Colormap = ...,
    norm: Normalize = ...,
    arrowsize: float = ...,
    arrowstyle: str = ...,
    minlength: float = ...,
    transform=...,
    zorder: int = ...,
    start_points=...,
    maxlength: float = ...,
    integration_direction: Literal["forward", "backward", "both"] = ...,
    broken_streamlines: bool = ...,
    *,
    data=...
) -> StreamplotSet: ...
def table(
    cellText=...,
    cellColours=...,
    cellLoc: Literal["left", "center", "right"] = ...,
    colWidths: list[float] = ...,
    rowLabels: list[str] = ...,
    rowColours: list[Color] = ...,
    rowLoc: Literal["left", "center", "right"] = ...,
    colLabels: list[str] = ...,
    colColours: list[Color] = ...,
    colLoc: Literal["left", "center", "right"] = ...,
    loc: str = ...,
    bbox: Bbox = ...,
    edges: Literal["open", "closed", "horizontal", "vertical"] = ...,
    **kwargs
) -> Table: ...
def text(x: float, y: float, s: str, fontdict: dict = ..., **kwargs) -> Text: ...
def tick_params(axis: Literal["x", "y", "both"] = ..., **kwargs): ...
def ticklabel_format(
    *,
    axis: Literal["x", "y", "both"] = ...,
    style: Literal["sci", "scientific", "plain"] = ...,
    scilimits=...,
    useOffset: bool | float = ...,
    useLocale: bool = ...,
    useMathText: bool = ...
): ...
def tricontour(*args, **kwargs) -> TriContourSet: ...
def tricontourf(*args, **kwargs) -> TriContourSet: ...
def tripcolor(
    *args,
    alpha=...,
    norm=...,
    cmap=...,
    vmin: float = ...,
    vmax: float = ...,
    shading: Literal["flat", "gouraud"] = ...,
    facecolors: ArrayLike = ...,
    **kwargs
): ...
def triplot(*args, **kwargs): ...
def violinplot(
    dataset: ArrayLike,
    positions: ArrayLike = ...,
    vert: bool = ...,
    widths: ArrayLike = ...,
    showmeans: bool = ...,
    showextrema: bool = ...,
    showmedians: bool = ...,
    quantiles: ArrayLike = ...,
    points: int = ...,
    bw_method: str | Scalar | Callable = ...,
    *,
    data=...
) -> dict: ...
def vlines(
    x: float | ArrayLike,
    ymin: float | ArrayLike,
    ymax: float | ArrayLike,
    colors: list[Color] = ...,
    linestyles: Literal["solid", "dashed", "dashdot", "dotted"] = ...,
    label: str = ...,
    *,
    data=...,
    **kwargs
) -> LineCollection: ...
def xcorr(
    x,
    y,
    normed: bool = ...,
    detrend: Callable = ...,
    usevlines: bool = ...,
    maxlags: int = ...,
    *,
    data=...,
    **kwargs
): ...
def sci(im): ...
def title(
    label: str,
    fontdict: dict = ...,
    loc: Literal["center", "left", "right"] = ...,
    pad: float = ...,
    *,
    y: float = ...,
    **kwargs
) -> Text: ...
def xlabel(
    xlabel: str,
    fontdict=...,
    labelpad: float = ...,
    *,
    loc: Literal["left", "center", "right"] = ...,
    **kwargs
): ...
def ylabel(
    ylabel: str,
    fontdict=...,
    labelpad: float = ...,
    *,
    loc: Literal["bottom", "center", "top"] = ...,
    **kwargs
): ...
def xscale(
    value: Literal["linear", "log", "symlog", "logit"] | ScaleBase, **kwargs
): ...
def yscale(
    value: Literal["linear", "log", "symlog", "logit"] | ScaleBase, **kwargs
): ...
def autumn(): ...
def bone(): ...
def cool(): ...
def copper(): ...
def flag(): ...
def gray(): ...
def hot(): ...
def hsv(): ...
def jet(): ...
def pink(): ...
def prism(): ...
def spring(): ...
def summer(): ...
def winter(): ...
def magma(): ...
def inferno(): ...
def plasma(): ...
def viridis(): ...
def nipy_spectral(): ...



from ..scene.node import Node
from ..gloo.framebuffer import FrameBuffer
from ..util.event import Event
from numpy.typing import ArrayLike
from ..visuals.visual import Visual
from typing import Mapping
from ..gloo.context import GLContext
from ..scene.widgets.widget import Widget
from ..util.svg.color import Color
from ..app.application import Application
from ..app.canvas import Canvas, DrawEvent

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import weakref
import numpy as np

from .. import gloo
from .. import app
from .visuals import VisualNode
from ..visuals.transforms import TransformSystem
from ..color import Color
from ..util import logger, Frozen
from ..util.profiler import Profiler
from .subscene import SubScene
from .events import SceneMouseEvent
from .widgets import Widget

class SceneCanvas(app.Canvas, Frozen):
    def __init__(
        self,
        title: str = "VisPy canvas",
        size: tuple[float, float] = ...,
        position: tuple[float, float] | None = None,
        show: bool = False,
        autoswap: bool = True,
        app: str | Application | None = None,
        create_native: bool = True,
        vsync: bool = False,
        resizable: bool = True,
        decorate: bool = True,
        fullscreen: bool | int = False,
        config: Mapping | None = None,
        shared: None | Canvas | GLContext = None,
        keys: str | Mapping | None = None,
        parent: Widget | None = None,
        dpi: None | float = None,
        always_on_top: bool = False,
        px_scale: int = 1,
        bgcolor: Color | str = "black",
    ): ...
    @property
    def scene(self): ...
    @scene.setter
    def scene(self, node): ...
    @property
    def central_widget(self): ...
    @property
    def bgcolor(self): ...
    @bgcolor.setter
    def bgcolor(self, color): ...
    def update(self, node: Node | None = None): ...
    def on_draw(self, event: Event): ...
    def render(
        self,
        region: tuple | None | None = None,
        size: tuple | None | None = None,
        bgcolor: None | Color = None,
        crop: ArrayLike | None = None,
        alpha: bool = True,
    ) -> ArrayLike: ...
    def _draw_scene(self, bgcolor=None): ...
    def draw_visual(self, visual: Visual, event: DrawEvent | None = None): ...
    def _generate_draw_order(self, node=None): ...
    def _update_scenegraph(self, event): ...
    def _process_mouse_event(self, event): ...
    def visual_at(self, pos: tuple) -> None | Visual: ...
    def _visual_bounds_at(self, pos, node=None): ...
    def visuals_at(self, pos: tuple, radius: int = 10): ...
    def _render_picking(self, crop): ...
    def on_resize(self, event: Event): ...
    def on_close(self, event: Event): ...

    # -------------------------------------------------- transform handling ---
    def push_viewport(self, viewport: tuple): ...
    def pop_viewport(self): ...
    def push_fbo(self, fbo: FrameBuffer, offset: tuple, csize: tuple): ...
    def pop_fbo(self): ...
    def _current_framebuffer(self): ...
    def _update_transforms(self): ...



from typing import Literal

from matplotlib.artist import Artist
from ._typing import *
from .collections import LineCollection
from .lines import Line2D
from .patches import Rectangle
from typing import Type

class Container(tuple):
    def __repr__(self) -> str: ...
    def __new__(cls: Type[Container], *args, **kwargs) -> Container: ...
    def __init__(self, kl: list[Rectangle], label: str = ...) -> None: ...
    def remove(self) -> None: ...
    def get_children(self) -> list[Rectangle]: ...

    get_label = Artist.get_label
    set_label = Artist.set_label
    add_callback = Artist.add_callback
    remove_callback = Artist.remove_callback
    pchanged = Artist.pchanged

class BarContainer(Container):

    patches: list[Rectangle]
    errorbar: None | ErrorbarContainer
    datavalues: None | ArrayLike
    orientation: None | Literal["horizontal", "vertical"]

    def __init__(
        self,
        patches: list[Rectangle],
        errorbar: ErrorbarContainer | None = ...,
        *,
        datavalues=...,
        orientation=...,
        **kwargs
    ) -> None: ...

class ErrorbarContainer(Container):

    lines: tuple[Line2D, tuple[Line2D, ...], list[LineCollection]]
    has_xerr: bool
    has_yerr: bool

    def __init__(
        self,
        lines: tuple[Line2D, tuple[Line2D, ...], list[LineCollection]],
        has_xerr: bool = ...,
        has_yerr: bool = ...,
        **kwargs
    ) -> None: ...

class StemContainer(Container):

    markerline: Line2D
    stemlines: list[Line2D]
    baseline: Line2D

    def __init__(
        self,
        markerline_stemlines_baseline: tuple[Line2D, list[Line2D], Line2D],
        **kwargs
    ) -> None: ...



from tkinter import Label
from matplotlib.collections import Collection
from typing import Callable, Iterable, Literal, Sequence
from ._typing import *
from .transforms import Transform
from .text import Text
from .cm import ScalarMappable
from .ticker import Formatter
from .axes import Axes
from .artist import Artist

class ClabelText(Text):
    def get_rotation(self) -> float: ...

class ContourLabeler:
    def clabel(
        self,
        levels: ArrayLike = ...,
        *,
        fontsize: str | float = ...,
        inline: bool = ...,
        inline_spacing: float = ...,
        fmt: Formatter | str | Callable | dict = ...,
        colors: Color | Sequence[Color] | None = ...,
        use_clabeltext: bool = ...,
        manual: bool | Iterable = ...,
        rightside_up: bool = ...,
        zorder: float | None = ...
    ) -> list[Label]: ...
    def print_label(self, linecontour, labelwidth) -> bool: ...
    def too_close(self, x, y, lw) -> bool: ...
    def get_label_width(self, lev, fmt, fsize) -> float: ...
    def set_label_props(self, label, text, color) -> None: ...
    def get_text(self, lev, fmt): ...
    def locate_label(self, linecontour, labelwidth): ...
    def calc_label_rot_and_inline(self, slc, ind, lw, lc=..., spacing=...): ...
    def add_label(self, x, y, rotation, lev, cvalue): ...
    def add_label_clabeltext(self, x, y, rotation, lev, cvalue): ...
    def add_label_near(
        self,
        x: float,
        y: float,
        inline: bool = ...,
        inline_spacing: int = ...,
        transform: Transform | Literal[False] = ...,
    ): ...
    def pop_label(self, index=...): ...
    def labels(self, inline, inline_spacing): ...

class ContourSet(ScalarMappable, ContourLabeler):
    def __init__(
        self,
        ax: Axes,
        *args,
        levels: Sequence[float] = ...,
        filled=...,
        linewidths=...,
        linestyles=...,
        hatches=...,
        alpha=...,
        origin=...,
        extent=...,
        cmap=...,
        colors=...,
        norm=...,
        vmin=...,
        vmax=...,
        extend=...,
        antialiased=...,
        nchunk=...,
        locator=...,
        transform=...,
        **kwargs
    ) -> None: ...
    def get_transform(self) -> Transform: ...
    def __getstate__(self): ...
    def legend_elements(
        self, variable_name: str = ..., str_format: Callable = ...
    ) -> tuple[list[Artist], list[str]]: ...
    def changed(self) -> bool: ...
    def get_alpha(self) -> float: ...
    def set_alpha(self, alpha: float) -> None: ...
    def find_nearest_contour(
        self, x: float, y: float, indices: list[int] | None = ..., pixel: bool = ...
    ) -> tuple[Collection, int, int, tuple[float, float], float]: ...

class QuadContourSet(ContourSet): ...



import string
from typing import Any, Dict, List, Optional, Tuple, Union

from django.http.request import HttpRequest

from .base import BaseEngine

class TemplateStrings(BaseEngine):
    template_dirs: Tuple[str]
    def __init__(
        self, params: Dict[str, Union[Dict[Any, Any], List[Any], bool, str]]
    ) -> None: ...

class Template(string.Template):
    template: str
    def render(
        self,
        context: Optional[Dict[str, str]] = ...,
        request: Optional[HttpRequest] = ...,
    ) -> str: ...



from numpy import ndarray, longlong

# Authors: William Mill (bill@billmill.org)
# License: BSD 3 clause

import numpy as np


class DrawTree:
    def __init__(
        self,
        tree: Tree,
        parent: DrawTree | None = None,
        depth: int = 0,
        number: int = 1,
    ) -> None:
        ...

    def left(self) -> DrawTree | int:
        ...

    def right(self) -> DrawTree | int:
        ...

    def lbrother(self) -> DrawTree | None:
        ...

    def get_lmost_sibling(self) -> DrawTree | None:
        ...

    lmost_sibling = ...

    def __str__(self) -> str:
        ...

    def __repr__(self) -> str:
        ...

    def max_extents(self) -> ndarray:
        ...


def buchheim(tree: Tree) -> DrawTree:
    ...


def third_walk(tree: DrawTree, n: float) -> None:
    ...


def first_walk(v: DrawTree, distance: float = 1.0) -> DrawTree:
    ...


def apportion(v: DrawTree, default_ancestor: DrawTree, distance: float) -> DrawTree:
    ...


def move_subtree(wl: DrawTree, wr: DrawTree, shift: float) -> None:
    ...


def execute_shifts(v: DrawTree) -> None:
    ...


def ancestor(vil: DrawTree, v: DrawTree, default_ancestor: DrawTree) -> DrawTree:
    ...


def second_walk(
    v: DrawTree, m: float | int = 0, depth: int = 0, min: float | None = None
) -> float:
    ...


class Tree:
    def __init__(
        self, label: str = "", node_id: longlong | int = ..., *children
    ) -> None:
        ...



import os
from collections import OrderedDict
from collections.abc import Iterator
from typing import Any

from transformers.configuration_utils import PretrainedConfig
from transformers.tokenization_utils_fast import PreTrainedTokenizerFast
from typing_extensions import TypeAlias

from .configuration_auto import AutoConfig as AutoConfig

_LazyAutoMappingValue: TypeAlias = tuple[
    # Tokenizers will depend on packages installed, too much variance and there are no common base or Protocol
    type[Any | None],
    type[PreTrainedTokenizerFast | None],
]

CLASS_DOCSTRING: str
FROM_CONFIG_DOCSTRING: str
FROM_PRETRAINED_TORCH_DOCSTRING: str
FROM_PRETRAINED_TF_DOCSTRING: str
FROM_PRETRAINED_FLAX_DOCSTRING: str

class _BaseAutoModelClass:
    def __init__(self, *args, **kwargs) -> None: ...
    @classmethod
    def from_config(cls, config, **kwargs): ...
    @classmethod
    def from_pretrained(cls, pretrained_model_name_or_path: str | os.PathLike[str], *model_args, **kwargs): ...
    @classmethod
    def register(cls, config_class, model_class) -> None: ...

def insert_head_doc(docstring, head_doc: str = ""): ...
def auto_class_update(cls, checkpoint_for_example: str = "bert-base-cased", head_doc: str = ""): ...
def get_values(model_mapping): ...
def getattribute_from_module(module, attr): ...

class _LazyAutoMapping(OrderedDict[type[PretrainedConfig], _LazyAutoMappingValue]):
    def __init__(self, config_mapping, model_mapping) -> None: ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: type[PretrainedConfig]) -> _LazyAutoMappingValue: ...
    def keys(self) -> list[type[PretrainedConfig]]: ...
    def get(self, key: type[PretrainedConfig], default: _LazyAutoMappingValue) -> _LazyAutoMappingValue: ...
    def __bool__(self) -> bool: ...
    def values(self) -> list[_LazyAutoMappingValue]: ...
    def items(self) -> list[tuple[type[PretrainedConfig], _LazyAutoMappingValue]]: ...
    def __iter__(self) -> Iterator[type[PretrainedConfig]]: ...
    def __contains__(self, item: object) -> bool: ...
    def register(self, key: type[PretrainedConfig], value: _LazyAutoMappingValue) -> None: ...



from numpy.typing import ArrayLike

from itertools import chain
from operator import add

import numpy as np

from ..draw import rectangle

FEATURE_TYPE = ...

def _validate_feature_type(feature_type): ...
def haar_like_feature_coord(
    width: int, height: int, feature_type: str | ArrayLike | None = None
): ...
def haar_like_feature(
    int_image,
    r: int,
    c: int,
    width: int,
    height: int,
    feature_type: str | ArrayLike | None = None,
    feature_coord=None,
): ...
def draw_haar_like_feature(
    image,
    r: int,
    c: int,
    width: int,
    height: int,
    feature_coord,
    color_positive_block: tuple[float, float, float] = ...,
    color_negative_block: tuple[float, float, float] = ...,
    alpha: float = 0.5,
    max_n_features: int | None = None,
    random_state=None,
): ...



from typing import ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy.special import gammainc as gammainc
from ..neighbors._base import VALID_METRICS as VALID_METRICS
from ._binary_tree import BinaryTree
from ._kd_tree import KDTree as KDTree
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from ..utils.extmath import row_norms as row_norms
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator
from ._ball_tree import BallTree as BallTree, DTYPE as DTYPE
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils import check_random_state as check_random_state
from ..utils.validation import check_is_fitted as check_is_fitted

KernelDensity_Self = TypeVar("KernelDensity_Self", bound="KernelDensity")

# Author: Jake Vanderplas <jakevdp@cs.washington.edu>
import itertools

import numpy as np


VALID_KERNELS: list = ...

TREE_DICT: dict = ...


# TODO: implement a brute force version for testing purposes
# TODO: create a density estimation base class?
class KernelDensity(BaseEstimator):
    bandwidth_: float = ...
    feature_names_in_: ndarray = ...
    tree_: BinaryTree = ...
    n_features_in_: int = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        bandwidth: float | Literal["scott", "silverman"] = 1.0,
        algorithm: Literal["kd_tree", "ball_tree", "auto", "auto"] = "auto",
        kernel: Literal[
            "gaussian",
            "tophat",
            "epanechnikov",
            "exponential",
            "linear",
            "cosine",
            "gaussian",
        ] = "gaussian",
        metric: str = "euclidean",
        atol: Float = 0,
        rtol: Float = 0,
        breadth_first: bool = True,
        leaf_size: Int = 40,
        metric_params: None | dict = None,
    ) -> None:
        ...

    def fit(
        self: KernelDensity_Self,
        X: MatrixLike,
        y=None,
        sample_weight: None | ArrayLike = None,
    ) -> KernelDensity_Self:
        ...

    def score_samples(self, X: MatrixLike) -> ndarray:
        ...

    def score(self, X: MatrixLike, y=None) -> Float:
        ...

    def sample(
        self, n_samples: Int = 1, random_state: RandomState | None | Int = None
    ) -> ndarray:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.special._comb, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__test__: dict
def _comb_int() -> typing.Any:
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Literal, Sequence
from ._typing import *
from .text import Text
from .path import Path
from .backend_bases import MouseEvent, RendererBase
from .font_manager import FontProperties
from .figure import Figure
from .transforms import Bbox, Transform
from .axes import Axes
from .artist import Artist, allow_rasterization
from .patches import Rectangle

class Cell(Rectangle):

    PAD = ...

    def __init__(
        self,
        xy: Sequence[float],
        width: float,
        height: float,
        edgecolor: Color = ...,
        facecolor: Color = ...,
        fill: bool = ...,
        text: str = ...,
        loc: Literal["left", "center", "right"] = "right",
        fontproperties: FontProperties = ...,
        *,
        visible_edges: str = "closed"
    ) -> None: ...
    def set_transform(self, trans: Transform): ...
    def set_figure(self, fig: Figure) -> None: ...
    def get_text(self) -> Text: ...
    def set_fontsize(self, size: float) -> None: ...
    def get_fontsize(self) -> float: ...
    def auto_set_font_size(self, renderer: RendererBase) -> float: ...
    @allow_rasterization
    def draw(self, renderer):
        RendererBase: ...
    def get_text_bounds(self, renderer: RendererBase): ...
    def get_required_width(self, renderer: RendererBase): ...
    def set_text_props(self, **kwargs) -> None: ...
    @property
    def visible_edges(self) -> str: ...
    @visible_edges.setter
    def visible_edges(self, value: str): ...
    def get_path(self) -> Path: ...

CustomCell = Cell

class Table(Artist):

    codes = ...
    FONTSIZE = ...
    AXESPAD = ...
    def __init__(
        self, ax: Axes, loc: str = ..., bbox: Bbox = ..., **kwargs
    ) -> None: ...
    def add_cell(self, row: int, col: int, *args, **kwargs) -> Cell: ...
    def __setitem__(self, position: Sequence[int], cell: Cell) -> None: ...
    def __getitem__(self, position) -> Cell: ...
    @property
    def edges(self): ...
    @edges.setter
    def edges(self, value): ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def contains(self, mouseevent: MouseEvent): ...
    def get_children(self) -> list[Artist]: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def auto_set_column_width(self, col: int) -> None: ...
    def auto_set_font_size(self, value: float = ...): ...
    def scale(self, xscale: float, yscale: float): ...
    def set_fontsize(self, size: float): ...
    def get_celld(self) -> dict[Sequence[int], Cell]: ...

def table(
    ax,
    cellText: ArrayLike = ...,
    cellColours: ArrayLike = ...,
    cellLoc: Literal["left", "center", "right"] = "right",
    colWidths: Sequence[float] = ...,
    rowLabels: Sequence[str] = ...,
    rowColours: Sequence[Color] = ...,
    rowLoc: Literal["left", "center", "right"] = "left",
    colLabels: Sequence[str] = ...,
    colColours: Sequence[Color] = ...,
    colLoc: Literal["left", "center", "right"] = "left",
    loc: str = ...,
    bbox: Bbox = ...,
    edges: str | Literal["open", "closed", "horizontal", "vertical"] = ...,
    **kwargs
) -> Table: ...



from ..classes.digraph import DiGraph
from numpy.typing import ArrayLike
from ..classes.graph import Graph
from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["modularity_matrix", "directed_modularity_matrix"]

def modularity_matrix(G: Graph, nodelist: ArrayLike | None = None, weight=None): ...
def directed_modularity_matrix(
    G: DiGraph, nodelist: ArrayLike | None = None, weight=None
): ...



from ._gpr import GaussianProcessRegressor as GaussianProcessRegressor
from ._gpc import GaussianProcessClassifier as GaussianProcessClassifier
from . import kernels as kernels

# Author: Jan Hendrik Metzen <jhm@informatik.uni-bremen.de>
#         Vincent Dubourg <vincent.dubourg@gmail.com>
#         (mostly translation, see implementation details)
# License: BSD 3 clause


__all__ = ["GaussianProcessRegressor", "GaussianProcessClassifier", "kernels"]




from abc import abstractmethod

import numpy as np

from typing import Any, Literal

from scipy.sparse import csr_matrix, spmatrix


def sqeuclidean_row_norms(X: np.ndarray|csr_matrix, num_threads: int) -> np.ndarray:
    """Compute the squared euclidean norm of the rows of X in parallel.

    Parameters
    ----------
    X : ndarray or CSR matrix of shape (n_samples, n_features)
        Input data. Must be c-contiguous.

    num_threads : int
        The number of OpenMP threads to use.

    Returns
    -------
    sqeuclidean_row_norms : ndarray of shape (n_samples,)
        Arrays containing the squared euclidean norm of each row of X.
    """
    ...


class BaseDistancesReductionDispatcher:
    """Abstract base dispatcher for pairwise distance computation & reduction.

    Each dispatcher extending the base :class:`BaseDistancesReductionDispatcher`
    dispatcher must implement the :meth:`compute` classmethod.
    """

    @classmethod
    def valid_metrics(cls) -> list[str]: ...

    @classmethod
    def is_usable_for(cls, X: np.ndarray|spmatrix, Y: np.ndarray|spmatrix, metric: str = "euclidean") -> bool:
        """Return True if the dispatcher can be used for the
        given parameters.

        Parameters
        ----------
        X : {ndarray, sparse matrix} of shape (n_samples_X, n_features)
            Input data.

        Y : {ndarray, sparse matrix} of shape (n_samples_Y, n_features)
            Input data.

        metric : str, default='euclidean'
            The distance metric to use.
            For a list of available metrics, see the documentation of
            :class:`~sklearn.metrics.DistanceMetric`.

        Returns
        -------
        True if the dispatcher can be used, else False.
        """
        ...

    @classmethod
    @abstractmethod
    def compute(
        cls,
        X: np.ndarray|csr_matrix,
        Y: np.ndarray|csr_matrix,
        **kwargs,
    ): ...



class ArgKmin(BaseDistancesReductionDispatcher):
    """Compute the argkmin of row vectors of X on the ones of Y.

    For each row vector of X, computes the indices of k first the rows
    vectors of Y with the smallest distances.

    ArgKmin is typically used to perform
    bruteforce k-nearest neighbors queries.

    This class is not meant to be instanciated, one should only use
    its :meth:`compute` classmethod which handles allocation and
    deallocation consistently.
    """

    @classmethod
    def compute(
        cls,
        X: np.ndarray|csr_matrix,
        Y: np.ndarray|csr_matrix,
        k: int,
        metric: str = "euclidean",
        chunk_size: int|None = None,
        metric_kwargs: dict[str,Any]|None = None,
        strategy: Literal['auto', 'parallel_on_X', 'parallel_on_Y']|None = None,
        return_distance: bool = False,
    ) -> np.ndarray | tuple[np.ndarray, np.ndarray]:
        """Compute the argkmin reduction.

        Parameters
        ----------
        X : ndarray or CSR matrix of shape (n_samples_X, n_features)
            Input data.

        Y : ndarray or CSR matrix of shape (n_samples_Y, n_features)
            Input data.

        k : int
            The k for the argkmin reduction.

        metric : str, default='euclidean'
            The distance metric to use for argkmin.
            For a list of available metrics, see the documentation of
            :class:`~sklearn.metrics.DistanceMetric`.

        chunk_size : int, default=None,
            The number of vectors per chunk. If None (default) looks-up in
            scikit-learn configuration for `pairwise_dist_chunk_size`,
            and use 256 if it is not set.

        metric_kwargs : dict, default=None
            Keyword arguments to pass to specified metric function.

        strategy : str, {'auto', 'parallel_on_X', 'parallel_on_Y'}, default=None
            The chunking strategy defining which dataset parallelization are made on.

            For both strategies the computations happens with two nested loops,
            respectively on chunks of X and chunks of Y.
            Strategies differs on which loop (outer or inner) is made to run
            in parallel with the Cython `prange` construct:

              - 'parallel_on_X' dispatches chunks of X uniformly on threads.
                Each thread then iterates on all the chunks of Y. This strategy is
                embarrassingly parallel and comes with no datastructures
                synchronisation.

              - 'parallel_on_Y' dispatches chunks of Y uniformly on threads.
                Each thread processes all the chunks of X in turn. This strategy is
                a sequence of embarrassingly parallel subtasks (the inner loop on Y
                chunks) with intermediate datastructures synchronisation at each
                iteration of the sequential outer loop on X chunks.

              - 'auto' relies on a simple heuristic to choose between
                'parallel_on_X' and 'parallel_on_Y': when `X.shape[0]` is large enough,
                'parallel_on_X' is usually the most efficient strategy.
                When `X.shape[0]` is small but `Y.shape[0]` is large, 'parallel_on_Y'
                brings more opportunity for parallelism and is therefore more efficient

              - None (default) looks-up in scikit-learn configuration for
                `pairwise_dist_parallel_strategy`, and use 'auto' if it is not set.

        return_distance : boolean, default=False
            Return distances between each X vector and its
            argkmin if set to True.

        Returns
        -------
        If return_distance=False:
          - argkmin_indices : ndarray of shape (n_samples_X, k)
            Indices of the argkmin for each vector in X.

        If return_distance=True:
          - argkmin_distances : ndarray of shape (n_samples_X, k)
            Distances to the argkmin for each vector in X.
          - argkmin_indices : ndarray of shape (n_samples_X, k)
            Indices of the argkmin for each vector in X.

        Notes
        -----
        This classmethod inspects the arguments values to dispatch to the
        dtype-specialized implementation of :class:`ArgKmin`.

        This allows decoupling the API entirely from the implementation details
        whilst maintaining RAII: all temporarily allocated datastructures necessary
        for the concrete implementation are therefore freed when this classmethod
        returns.
        """
        ...


class RadiusNeighbors(BaseDistancesReductionDispatcher):
    """Compute radius-based neighbors for two sets of vectors.

    For each row-vector X[i] of the queries X, find all the indices j of
    row-vectors in Y such that:

                        dist(X[i], Y[j]) <= radius

    The distance function `dist` depends on the values of the `metric`
    and `metric_kwargs` parameters.

    This class is not meant to be instanciated, one should only use
    its :meth:`compute` classmethod which handles allocation and
    deallocation consistently.
    """

    @classmethod
    def compute(
        cls,
        X: np.ndarray | csr_matrix,
        Y: np.ndarray | csr_matrix,
        radius: float,
        metric: str = "euclidean",
        chunk_size: int | None = None,
        metric_kwargs: dict[str,Any] | None = None,
        strategy: Literal['auto', 'parallel_on_X', 'parallel_on_Y']|None = None,
        return_distance: bool = False,
        sort_results: bool = False,
    ):
        """Return the results of the reduction for the given arguments.

        Parameters
        ----------
        X : ndarray or CSR matrix of shape (n_samples_X, n_features)
            Input data.

        Y : ndarray or CSR matrix of shape (n_samples_Y, n_features)
            Input data.

        radius : float
            The radius defining the neighborhood.

        metric : str, default='euclidean'
            The distance metric to use.
            For a list of available metrics, see the documentation of
            :class:`~sklearn.metrics.DistanceMetric`.

        chunk_size : int, default=None,
            The number of vectors per chunk. If None (default) looks-up in
            scikit-learn configuration for `pairwise_dist_chunk_size`,
            and use 256 if it is not set.

        metric_kwargs : dict, default=None
            Keyword arguments to pass to specified metric function.

        strategy : str, {'auto', 'parallel_on_X', 'parallel_on_Y'}, default=None
            The chunking strategy defining which dataset parallelization are made on.

            For both strategies the computations happens with two nested loops,
            respectively on chunks of X and chunks of Y.
            Strategies differs on which loop (outer or inner) is made to run
            in parallel with the Cython `prange` construct:

              - 'parallel_on_X' dispatches chunks of X uniformly on threads.
                Each thread then iterates on all the chunks of Y. This strategy is
                embarrassingly parallel and comes with no datastructures
                synchronisation.

              - 'parallel_on_Y' dispatches chunks of Y uniformly on threads.
                Each thread processes all the chunks of X in turn. This strategy is
                a sequence of embarrassingly parallel subtasks (the inner loop on Y
                chunks) with intermediate datastructures synchronisation at each
                iteration of the sequential outer loop on X chunks.

              - 'auto' relies on a simple heuristic to choose between
                'parallel_on_X' and 'parallel_on_Y': when `X.shape[0]` is large enough,
                'parallel_on_X' is usually the most efficient strategy.
                When `X.shape[0]` is small but `Y.shape[0]` is large, 'parallel_on_Y'
                brings more opportunity for parallelism and is therefore more efficient
                despite the synchronization step at each iteration of the outer loop
                on chunks of `X`.

              - None (default) looks-up in scikit-learn configuration for
                `pairwise_dist_parallel_strategy`, and use 'auto' if it is not set.

        return_distance : boolean, default=False
            Return distances between each X vector and its neighbors if set to True.

        sort_results : boolean, default=False
            Sort results with respect to distances between each X vector and its
            neighbors if set to True.

        Returns
        -------
        If return_distance=False:
          - neighbors_indices : ndarray of n_samples_X ndarray
            Indices of the neighbors for each vector in X.

        If return_distance=True:
          - neighbors_indices : ndarray of n_samples_X ndarray
            Indices of the neighbors for each vector in X.
          - neighbors_distances : ndarray of n_samples_X ndarray
            Distances to the neighbors for each vector in X.

        Notes
        -----
        This classmethod inspects the arguments values to dispatch to the
        dtype-specialized implementation of :class:`RadiusNeighbors`.

        This allows decoupling the API entirely from the implementation details
        whilst maintaining RAII: all temporarily allocated datastructures necessary
        for the concrete implementation are therefore freed when this classmethod
        returns.
        """
        ...




from ..geometry.meshdata import MeshData
from numpy.typing import ArrayLike, NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
# Author: Nicolas P .Rougier
# Date:   04/03/2014
# -----------------------------------------------------------------------------

import numpy as np

from .meshdata import MeshData

def create_cube() -> tuple[ArrayLike, ArrayLike, ArrayLike]: ...
def create_plane(
    width: float = 1,
    height: float = 1,
    width_segments: int = 1,
    height_segments: float = 1,
    direction: str = "+z",
) -> tuple[ArrayLike, ArrayLike, ArrayLike]: ...
def create_box(
    width: float = 1,
    height: float = 1,
    depth: float = 1,
    width_segments: int = 1,
    height_segments: float = 1,
    depth_segments: float = 1,
    planes: ArrayLike | None = None,
) -> tuple[ArrayLike, ArrayLike, ArrayLike]: ...
def _latitude(rows, cols, radius, offset): ...
def _ico(radius, subdivisions): ...
def _cube(rows, cols, depth, radius): ...
def create_sphere(
    rows: int = 10,
    cols: int = 10,
    depth: int = 10,
    radius: float = 1.0,
    offset: bool = True,
    subdivisions: int = 3,
    method: str = "latitude",
) -> MeshData: ...
def create_cylinder(
    rows: int,
    cols: int,
    radius: tuple[float, ...] = ...,
    length: float = 1.0,
    offset: bool = False,
) -> MeshData: ...
def create_cone(cols: int, radius: float = 1.0, length: float = 1.0) -> MeshData: ...
def create_arrow(
    rows: int,
    cols: int,
    radius: float = 0.1,
    length: float = 1.0,
    cone_radius: float | None = None,
    cone_length: float | None = None,
) -> MeshData: ...
def create_grid_mesh(xs: NDArray, ys: NDArray, zs: NDArray) -> tuple[NDArray, NDArray]: ...



from typing import Callable
from numpy.typing import ArrayLike

__all__ = ["forest_str"]

def forest_str(
    graph,
    with_labels: bool = True,
    sources: ArrayLike | None = None,
    write: Callable | None = None,
    ascii_only: bool = False,
): ...



from typing import Literal, Sequence
from .._typing import Float, Int
from numpy import ndarray

# Authors: Jiyuan Qian <jq401@nyu.edu>
# License: BSD 3 clause

import numpy as np


class BaseOptimizer:
    learning_rate: float = ...

    def __init__(self, learning_rate_init: Float = 0.1) -> None:
        ...

    def update_params(
        self, params: Sequence | list[ndarray], grads: Sequence | list[ndarray]
    ) -> None:
        ...

    def iteration_ends(self, time_step: int) -> None:
        ...

    def trigger_stopping(self, msg: str, verbose: bool) -> bool:
        ...


class SGDOptimizer(BaseOptimizer):
    velocities: list = ...
    learning_rate: float = ...

    def __init__(
        self,
        params: Sequence | list[ndarray],
        learning_rate_init: Float = 0.1,
        lr_schedule: Literal[
            "constant", "adaptive", "invscaling", "constant"
        ] = "constant",
        momentum: Float = 0.9,
        nesterov: bool = True,
        power_t: Float = 0.5,
    ) -> None:
        ...

    def iteration_ends(self, time_step: Int) -> None:
        ...

    def trigger_stopping(self, msg: str, verbose: bool) -> bool:
        ...


class AdamOptimizer(BaseOptimizer):
    vs: list = ...
    ms: list = ...
    t: int = ...
    learning_rate: float = ...

    def __init__(
        self,
        params: Sequence | list[ndarray],
        learning_rate_init: Float = 0.001,
        beta_1: Float = 0.9,
        beta_2: Float = 0.999,
        epsilon: Float = 1e-8,
    ) -> None:
        ...



from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["mycielskian", "mycielski_graph"]

def mycielskian(G: Graph, iterations: int = 1): ...
def mycielski_graph(n: int): ...



import typing


def lobpcg(
    A,
    X,
    B=...,
    M=...,
    Y=...,
    tol=...,
    maxiter=...,
    largest=...,
    verbosityLevel=...,
    retLambdaHistory=...,
    retResidualNormsHistory=...,
)-> typing.Any:
        """Locally Optimal Block Preconditioned Conjugate Gradient Method (LOBPCG)

        LOBPCG is a preconditioned eigensolver for large symmetric positive
        definite (SPD) generalized eigenproblems.

        Parameters
        ----------
        A : {sparse matrix, dense matrix, LinearOperator}
            The symmetric linear operator of the problem, usually a
            sparse matrix.  Often called the "stiffness matrix".
        X : ndarray, float32 or float64
            Initial approximation to the ``k`` eigenvectors (non-sparse). If `A`
            has ``shape=(n,n)`` then `X` should have shape ``shape=(n,k)``.
        B : {dense matrix, sparse matrix, LinearOperator}, optional
            The right hand side operator in a generalized eigenproblem.
            By default, ``B = Identity``.  Often called the "mass matrix".
        M : {dense matrix, sparse matrix, LinearOperator}, optional
            Preconditioner to `A`; by default ``M = Identity``.
            `M` should approximate the inverse of `A`.
        Y : ndarray, float32 or float64, optional
            n-by-sizeY matrix of constraints (non-sparse), sizeY < n
            The iterations will be performed in the B-orthogonal complement
            of the column-space of Y. Y must be full rank.
        tol : scalar, optional
            Solver tolerance (stopping criterion).
            The default is ``tol=n*sqrt(eps)``.
        maxiter : int, optional
            Maximum number of iterations.  The default is ``maxiter = 20``.
        largest : bool, optional
            When True, solve for the largest eigenvalues, otherwise the smallest.
        verbosityLevel : int, optional
            Controls solver output.  The default is ``verbosityLevel=0``.
        retLambdaHistory : bool, optional
            Whether to return eigenvalue history.  Default is False.
        retResidualNormsHistory : bool, optional
            Whether to return history of residual norms.  Default is False.

        Returns
        -------
        w : ndarray
            Array of ``k`` eigenvalues
        v : ndarray
            An array of ``k`` eigenvectors.  `v` has the same shape as `X`.
        lambdas : list of ndarray, optional
            The eigenvalue history, if `retLambdaHistory` is True.
        rnorms : list of ndarray, optional
            The history of residual norms, if `retResidualNormsHistory` is True.

        Notes
        -----
        If both ``retLambdaHistory`` and ``retResidualNormsHistory`` are True,
        the return tuple has the following format
        ``(lambda, V, lambda history, residual norms history)``.

        In the following ``n`` denotes the matrix size and ``m`` the number
        of required eigenvalues (smallest or largest).

        The LOBPCG code internally solves eigenproblems of the size ``3m`` on every
        iteration by calling the "standard" dense eigensolver, so if ``m`` is not
        small enough compared to ``n``, it does not make sense to call the LOBPCG
        code, but rather one should use the "standard" eigensolver, e.g. numpy or
        scipy function in this case.
        If one calls the LOBPCG algorithm for ``5m > n``, it will most likely break
        internally, so the code tries to call the standard function instead.

        It is not that ``n`` should be large for the LOBPCG to work, but rather the
        ratio ``n / m`` should be large. It you call LOBPCG with ``m=1``
        and ``n=10``, it works though ``n`` is small. The method is intended
        for extremely large ``n / m``.

        The convergence speed depends basically on two factors:

        1. How well relatively separated the seeking eigenvalues are from the rest
        of the eigenvalues. One can try to vary ``m`` to make this better.

        2. How well conditioned the problem is. This can be changed by using proper
        preconditioning. For example, a rod vibration test problem (under tests
        directory) is ill-conditioned for large ``n``, so convergence will be
        slow, unless efficient preconditioning is used. For this specific
        problem, a good simple preconditioner function would be a linear solve
        for `A`, which is easy to code since A is tridiagonal.

        References
        ----------
        .. [1] A. V. Knyazev (2001),
            Toward the Optimal Preconditioned Eigensolver: Locally Optimal
            Block Preconditioned Conjugate Gradient Method.
            SIAM Journal on Scientific Computing 23, no. 2,
            pp. 517-541. :doi:`10.1137/S1064827500366124`

        .. [2] A. V. Knyazev, I. Lashuk, M. E. Argentati, and E. Ovchinnikov
            (2007), Block Locally Optimal Preconditioned Eigenvalue Xolvers
            (BLOPEX) in hypre and PETSc. :arxiv:`0705.2626`

        .. [3] A. V. Knyazev's C and MATLAB implementations:
            https://github.com/lobpcg/blopex

        Examples
        --------

        Solve ``A x = lambda x`` with constraints and preconditioning.

        >>> import numpy as np
        >>> from scipy.sparse import spdiags, issparse
        >>> from scipy.sparse.linalg import lobpcg, LinearOperator
        >>> n = 100
        >>> vals = np.arange(1, n + 1)
        >>> A = spdiags(vals, 0, n, n)
        >>> A.toarray()
        array([[  1.,   0.,   0., ...,   0.,   0.,   0.],
            [  0.,   2.,   0., ...,   0.,   0.,   0.],
            [  0.,   0.,   3., ...,   0.,   0.,   0.],
            ...,
            [  0.,   0.,   0., ...,  98.,   0.,   0.],
            [  0.,   0.,   0., ...,   0.,  99.,   0.],
            [  0.,   0.,   0., ...,   0.,   0., 100.]])

        Constraints:

        >>> Y = np.eye(n, 3)

        Initial guess for eigenvectors, should have linearly independent
        columns. Column dimension = number of requested eigenvalues.

        >>> rng = np.random.default_rng()
        >>> X = rng.random((n, 3))

        Preconditioner in the inverse of A in this example:

        >>> invA = spdiags([1./vals], 0, n, n)

        The preconditiner must be defined by a function:

        >>> def precond( x ):
        ...     return invA @ x

        The argument x of the preconditioner function is a matrix inside `lobpcg`,
        thus the use of matrix-matrix product ``@``.

        The preconditioner function is passed to lobpcg as a `LinearOperator`:

        >>> M = LinearOperator(matvec=precond, matmat=precond,
        ...                    shape=(n, n), dtype=np.float64)

        Let us now solve the eigenvalue problem for the matrix A:

        >>> eigenvalues, _ = lobpcg(A, X, Y=Y, M=M, largest=False)
        >>> eigenvalues
        array([4., 5., 6.])

        Note that the vectors passed in Y are the eigenvectors of the 3 smallest
        eigenvalues. The results returned are orthogonal to those.

        """
        ...


from typing import ClassVar, Iterable, Literal, TypeVar
from ..model_selection import BaseCrossValidator
from scipy import linalg as linalg
from ._base import LinearModel
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from ..model_selection._split import BaseShuffleSplit
from numbers import Integral as Integral, Real as Real
from scipy.linalg.lapack import get_lapack_funcs as get_lapack_funcs
from math import sqrt as sqrt
from ..base import RegressorMixin, MultiOutputMixin
from ..model_selection import check_cv as check_cv
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import ArrayLike, MatrixLike, Int, Float
from ..utils import as_float_array as as_float_array, check_array as check_array

OrthogonalMatchingPursuitCV_Self = TypeVar(
    "OrthogonalMatchingPursuitCV_Self", bound="OrthogonalMatchingPursuitCV"
)
OrthogonalMatchingPursuit_Self = TypeVar(
    "OrthogonalMatchingPursuit_Self", bound="OrthogonalMatchingPursuit"
)


# Author: Vlad Niculae
#
# License: BSD 3 clause

import warnings
import numpy as np

premature: str = ...


def orthogonal_mp(
    X: ArrayLike,
    y: MatrixLike | ArrayLike,
    *,
    n_nonzero_coefs: None | Int = None,
    tol: None | Float = None,
    precompute: str | bool = False,
    copy_X: bool = True,
    return_path: bool = False,
    return_n_iter: bool = False,
) -> ndarray | tuple[ndarray, int] | tuple[ndarray, ndarray | int]:
    ...


def orthogonal_mp_gram(
    Gram: MatrixLike,
    Xy: MatrixLike | ArrayLike,
    *,
    n_nonzero_coefs: None | Int = None,
    tol: None | Float = None,
    norms_squared: None | ArrayLike = None,
    copy_Gram: bool = True,
    copy_Xy: bool = True,
    return_path: bool = False,
    return_n_iter: bool = False,
) -> ndarray | tuple[ndarray, ndarray | int]:
    ...


class OrthogonalMatchingPursuit(MultiOutputMixin, RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_nonzero_coefs_: int = ...
    n_iter_: ArrayLike | int = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        n_nonzero_coefs: None | Int = None,
        tol: None | Float = None,
        fit_intercept: bool = True,
        normalize: str | bool = "deprecated",
        precompute: Literal["auto", "auto"] | bool = "auto",
    ) -> None:
        ...

    def fit(
        self: OrthogonalMatchingPursuit_Self, X: MatrixLike, y: MatrixLike | ArrayLike
    ) -> OrthogonalMatchingPursuit_Self:
        ...


class OrthogonalMatchingPursuitCV(RegressorMixin, LinearModel):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: ArrayLike | int = ...
    n_nonzero_coefs_: int = ...
    coef_: ndarray = ...
    intercept_: float | ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        copy: bool = True,
        fit_intercept: bool = True,
        normalize: str | bool = "deprecated",
        max_iter: None | Int = None,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        n_jobs: None | Int = None,
        verbose: int | bool = False,
    ) -> None:
        ...

    def fit(
        self: OrthogonalMatchingPursuitCV_Self, X: MatrixLike, y: ArrayLike
    ) -> OrthogonalMatchingPursuitCV_Self:
        ...



from typing import Mapping
from .flow_matrix import flow_matrix_row
from ...classes.graph import Graph
from ...utils import not_implemented_for, reverse_cuthill_mckee_ordering

__all__ = [
    "current_flow_betweenness_centrality_subset",
    "edge_current_flow_betweenness_centrality_subset",
]

def current_flow_betweenness_centrality_subset(
    G: Graph, sources, targets, normalized=True, weight=None, dtype=..., solver="lu"
) -> Mapping: ...
def edge_current_flow_betweenness_centrality_subset(
    G: Graph, sources, targets, normalized=True, weight=None, dtype=..., solver="lu"
) -> Mapping: ...



from ..classes.graph import Graph
from ..exception import NetworkXNoPath

from ..utils import not_implemented_for

__all__ = ["efficiency", "local_efficiency", "global_efficiency"]

def efficiency(G: Graph, u, v) -> float: ...
def global_efficiency(G: Graph) -> float: ...
def local_efficiency(G: Graph) -> float: ...



from typing import Mapping
from collections import Counter
from itertools import combinations, repeat

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "is_matching",
    "is_maximal_matching",
    "is_perfect_matching",
    "max_weight_matching",
    "min_weight_matching",
    "maximal_matching",
]

def maximal_matching(G: Graph) -> set: ...
def matching_dict_to_set(matching): ...
def is_matching(G: Graph, matching: Mapping | set) -> bool: ...
def is_maximal_matching(G: Graph, matching: Mapping | set) -> bool: ...
def is_perfect_matching(G: Graph, matching: Mapping | set) -> bool: ...
def min_weight_matching(
    G: Graph, maxcardinality: bool | None = None, weight="weight"
) -> set: ...
def max_weight_matching(G: Graph, maxcardinality=False, weight="weight") -> set: ...



__all__ = [
    "HasACycle",
    "NodeNotFound",
    "PowerIterationFailedConvergence",
    "ExceededMaxIterations",
    "AmbiguousSolution",
    "NetworkXAlgorithmError",
    "NetworkXException",
    "NetworkXError",
    "NetworkXNoCycle",
    "NetworkXNoPath",
    "NetworkXNotImplemented",
    "NetworkXPointlessConcept",
    "NetworkXUnbounded",
    "NetworkXUnfeasible",
]

class NetworkXException(Exception):
    pass

class NetworkXError(NetworkXException):
    pass

class NetworkXPointlessConcept(NetworkXException):
    pass

class NetworkXAlgorithmError(NetworkXException):
    pass

class NetworkXUnfeasible(NetworkXAlgorithmError):
    pass

class NetworkXNoPath(NetworkXUnfeasible):
    pass

class NetworkXNoCycle(NetworkXUnfeasible):
    pass

class HasACycle(NetworkXException):
    pass

class NetworkXUnbounded(NetworkXAlgorithmError):
    pass

class NetworkXNotImplemented(NetworkXException):
    pass

class NodeNotFound(NetworkXException):
    pass

class AmbiguousSolution(NetworkXException):
    pass

class ExceededMaxIterations(NetworkXException):
    pass

class PowerIterationFailedConvergence(ExceededMaxIterations):
    def __init__(self, num_iterations, *args, **kw): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.linalg._decomp_update, version: unspecified
import typing
import builtins as _mod_builtins
import numpy.linalg as _mod_numpy_linalg

LinAlgError = _mod_numpy_linalg.LinAlgError
__all__: list
__doc__: str
__file__: str
__name__: str
__package__: str
__test__: dict
def _form_qTu() -> typing.Any:
    ' this function only exists to expose the cdef version below for testing\n        purposes. Here we perform minimal input validation to ensure that the\n        inputs meet the requirements below.\n    '
    ...

def qr_delete(Q, R, k, intp=..., which=..., overwrite_qr=..., check_finite=...) -> typing.Any:
    "qr_delete(Q, R, k, int p=1, which=u'row', overwrite_qr=False, check_finite=True)\n\n    QR downdate on row or column deletions\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A`` where ``p`` rows or columns have been removed\n    starting at row or column ``k``.\n\n    Parameters\n    ----------\n    Q : (M, M) or (M, N) array_like\n        Unitary/orthogonal matrix from QR decomposition.\n    R : (M, N) or (N, N) array_like\n        Upper triangular matrix from QR decomposition.\n    k : int\n        Index of the first row or column to delete.\n    p : int, optional\n        Number of rows or columns to delete, defaults to 1.\n    which: {'row', 'col'}, optional\n        Determines if rows or columns will be deleted, defaults to 'row'\n    overwrite_qr : bool, optional\n        If True, consume Q and R, overwriting their contents with their\n        downdated versions, and returning approriately sized views.\n        Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    See Also\n    --------\n    qr, qr_multiply, qr_insert, qr_update\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of ``R1`` are\n    positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this QR decomposition, update q and r when 2 rows are removed.\n\n    >>> q1, r1 = linalg.qr_delete(q, r, 2, 2, 'row', False)\n    >>> q1\n    array([[ 0.30942637,  0.15347579,  0.93845645],  # may vary (signs)\n           [ 0.61885275,  0.71680171, -0.32127338],\n           [ 0.72199487, -0.68017681, -0.12681844]])\n    >>> r1\n    array([[  9.69535971,  -0.4125685 ,  -6.80738023],  # may vary (signs)\n           [  0.        , -12.19958144,   1.62370412],\n           [  0.        ,   0.        ,  -0.15218213]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a1 = np.delete(a, slice(2,4), 0)\n    >>> a1\n    array([[ 3., -2., -2.],\n           [ 6., -9., -3.],\n           [ 7.,  8., -6.]])\n    >>> q_direct, r_direct = linalg.qr(a1)\n\n    Check that we have equivalent results:\n\n    >>> np.dot(q1, r1)\n    array([[ 3., -2., -2.],\n           [ 6., -9., -3.],\n           [ 7.,  8., -6.]])\n    >>> np.allclose(np.dot(q1, r1), a1)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q1.T, q1), np.eye(3))\n    True\n\n    "
    ...

def qr_insert(Q, R, u, k, which=..., rcond=..., overwrite_qru=..., check_finite=...) -> typing.Any:
    "qr_insert(Q, R, u, k, which=u'row', rcond=None, overwrite_qru=False, check_finite=True)\n\n    QR update on row or column insertions\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A`` where rows or columns have been inserted starting\n    at row or column ``k``.\n\n    Parameters\n    ----------\n    Q : (M, M) array_like\n        Unitary/orthogonal matrix from the QR decomposition of A.\n    R : (M, N) array_like\n        Upper triangular matrix from the QR decomposition of A.\n    u : (N,), (p, N), (M,), or (M, p) array_like\n        Rows or columns to insert\n    k : int\n        Index before which `u` is to be inserted.\n    which: {'row', 'col'}, optional\n        Determines if rows or columns will be inserted, defaults to 'row'\n    rcond : float\n        Lower bound on the reciprocal condition number of ``Q`` augmented with\n        ``u/||u||`` Only used when updating economic mode (thin, (M,N) (N,N))\n        decompositions.  If None, machine precision is used.  Defaults to\n        None.\n    overwrite_qru : bool, optional\n        If True, consume Q, R, and u, if possible, while performing the update,\n        otherwise make copies as necessary. Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrices contain only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    Raises\n    ------\n    LinAlgError :\n        If updating a (M,N) (N,N) factorization and the reciprocal condition\n        number of Q augmented with u/||u|| is smaller than rcond.\n\n    See Also\n    --------\n    qr, qr_multiply, qr_delete, qr_update\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of ``R1`` are\n    positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this QR decomposition, update q and r when 2 rows are inserted.\n\n    >>> u = np.array([[  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.]])\n    >>> q1, r1 = linalg.qr_insert(q, r, u, 2, 'row')\n    >>> q1\n    array([[-0.25445668,  0.02246245,  0.18146236, -0.72798806,  0.60979671],  # may vary (signs)\n           [-0.50891336,  0.23226178, -0.82836478, -0.02837033, -0.00828114],\n           [-0.50891336,  0.35715302,  0.38937158,  0.58110733,  0.35235345],\n           [ 0.25445668, -0.52202743, -0.32165498,  0.36263239,  0.65404509],\n           [-0.59373225, -0.73856549,  0.16065817, -0.0063658 , -0.27595554]])\n    >>> r1\n    array([[-11.78982612,   6.44623587,   3.81685018],  # may vary (signs)\n           [  0.        , -16.01393278,   3.72202865],\n           [  0.        ,   0.        ,  -6.13010256],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a1 = np.insert(a, 2, u, 0)\n    >>> a1\n    array([[  3.,  -2.,  -2.],\n           [  6.,  -7.,   4.],\n           [  6.,  -9.,  -3.],\n           [ -3.,  10.,   1.],\n           [  7.,   8.,  -6.]])\n    >>> q_direct, r_direct = linalg.qr(a1)\n\n    Check that we have equivalent results:\n\n    >>> np.dot(q1, r1)\n    array([[  3.,  -2.,  -2.],\n           [  6.,  -7.,   4.],\n           [  6.,  -9.,  -3.],\n           [ -3.,  10.,   1.],\n           [  7.,   8.,  -6.]])\n\n    >>> np.allclose(np.dot(q1, r1), a1)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q1.T, q1), np.eye(5))\n    True\n\n    "
    ...

def qr_update(Q, R, u, v, overwrite_qruv=..., check_finite=...) -> typing.Any:
    "qr_update(Q, R, u, v, overwrite_qruv=False, check_finite=True)\n\n    Rank-k QR update\n\n    If ``A = Q R`` is the QR factorization of ``A``, return the QR\n    factorization of ``A + u v**T`` for real ``A`` or ``A + u v**H``\n    for complex ``A``.\n\n    Parameters\n    ----------\n    Q : (M, M) or (M, N) array_like\n        Unitary/orthogonal matrix from the qr decomposition of A.\n    R : (M, N) or (N, N) array_like\n        Upper triangular matrix from the qr decomposition of A.\n    u : (M,) or (M, k) array_like\n        Left update vector\n    v : (N,) or (N, k) array_like\n        Right update vector\n    overwrite_qruv : bool, optional\n        If True, consume Q, R, u, and v, if possible, while performing the\n        update, otherwise make copies as necessary. Defaults to False.\n    check_finite : bool, optional\n        Whether to check that the input matrix contains only finite numbers.\n        Disabling may give a performance gain, but may result in problems\n        (crashes, non-termination) if the inputs do contain infinities or NaNs.\n        Default is True.\n\n    Returns\n    -------\n    Q1 : ndarray\n        Updated unitary/orthogonal factor\n    R1 : ndarray\n        Updated upper triangular factor\n\n    See Also\n    --------\n    qr, qr_multiply, qr_delete, qr_insert\n\n    Notes\n    -----\n    This routine does not guarantee that the diagonal entries of `R1` are\n    real or positive.\n\n    .. versionadded:: 0.16.0\n\n    References\n    ----------\n    .. [1] Golub, G. H. & Van Loan, C. F. Matrix Computations, 3rd Ed.\n           (Johns Hopkins University Press, 1996).\n\n    .. [2] Daniel, J. W., Gragg, W. B., Kaufman, L. & Stewart, G. W.\n           Reorthogonalization and stable algorithms for updating the\n           Gram-Schmidt QR factorization. Math. Comput. 30, 772-795 (1976).\n\n    .. [3] Reichel, L. & Gragg, W. B. Algorithm 686: FORTRAN Subroutines for\n           Updating the QR Decomposition. ACM Trans. Math. Softw. 16, 369-377\n           (1990).\n\n    Examples\n    --------\n    >>> from scipy import linalg\n    >>> a = np.array([[  3.,  -2.,  -2.],\n    ...               [  6.,  -9.,  -3.],\n    ...               [ -3.,  10.,   1.],\n    ...               [  6.,  -7.,   4.],\n    ...               [  7.,   8.,  -6.]])\n    >>> q, r = linalg.qr(a)\n\n    Given this q, r decomposition, perform a rank 1 update.\n\n    >>> u = np.array([7., -2., 4., 3., 5.])\n    >>> v = np.array([1., 3., -5.])\n    >>> q_up, r_up = linalg.qr_update(q, r, u, v, False)\n    >>> q_up\n    array([[ 0.54073807,  0.18645997,  0.81707661, -0.02136616,  0.06902409],  # may vary (signs)\n           [ 0.21629523, -0.63257324,  0.06567893,  0.34125904, -0.65749222],\n           [ 0.05407381,  0.64757787, -0.12781284, -0.20031219, -0.72198188],\n           [ 0.48666426, -0.30466718, -0.27487277, -0.77079214,  0.0256951 ],\n           [ 0.64888568,  0.23001   , -0.4859845 ,  0.49883891,  0.20253783]])\n    >>> r_up\n    array([[ 18.49324201,  24.11691794, -44.98940746],  # may vary (signs)\n           [  0.        ,  31.95894662, -27.40998201],\n           [  0.        ,   0.        ,  -9.25451794],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    The update is equivalent, but faster than the following.\n\n    >>> a_up = a + np.outer(u, v)\n    >>> q_direct, r_direct = linalg.qr(a_up)\n\n    Check that we have equivalent results:\n\n    >>> np.allclose(np.dot(q_up, r_up), a_up)\n    True\n\n    And the updated Q is still unitary:\n\n    >>> np.allclose(np.dot(q_up.T, q_up), np.eye(5))\n    True\n\n    Updating economic (reduced, thin) decompositions is also possible:\n\n    >>> qe, re = linalg.qr(a, mode='economic')\n    >>> qe_up, re_up = linalg.qr_update(qe, re, u, v, False)\n    >>> qe_up\n    array([[ 0.54073807,  0.18645997,  0.81707661],  # may vary (signs)\n           [ 0.21629523, -0.63257324,  0.06567893],\n           [ 0.05407381,  0.64757787, -0.12781284],\n           [ 0.48666426, -0.30466718, -0.27487277],\n           [ 0.64888568,  0.23001   , -0.4859845 ]])\n    >>> re_up\n    array([[ 18.49324201,  24.11691794, -44.98940746],  # may vary (signs)\n           [  0.        ,  31.95894662, -27.40998201],\n           [  0.        ,   0.        ,  -9.25451794]])\n    >>> np.allclose(np.dot(qe_up, re_up), a_up)\n    True\n    >>> np.allclose(np.dot(qe_up.T, qe_up), np.eye(3))\n    True\n\n    Similarly to the above, perform a rank 2 update.\n\n    >>> u2 = np.array([[ 7., -1,],\n    ...                [-2.,  4.],\n    ...                [ 4.,  2.],\n    ...                [ 3., -6.],\n    ...                [ 5.,  3.]])\n    >>> v2 = np.array([[ 1., 2.],\n    ...                [ 3., 4.],\n    ...                [-5., 2]])\n    >>> q_up2, r_up2 = linalg.qr_update(q, r, u2, v2, False)\n    >>> q_up2\n    array([[-0.33626508, -0.03477253,  0.61956287, -0.64352987, -0.29618884],  # may vary (signs)\n           [-0.50439762,  0.58319694, -0.43010077, -0.33395279,  0.33008064],\n           [-0.21016568, -0.63123106,  0.0582249 , -0.13675572,  0.73163206],\n           [ 0.12609941,  0.49694436,  0.64590024,  0.31191919,  0.47187344],\n           [-0.75659643, -0.11517748,  0.10284903,  0.5986227 , -0.21299983]])\n    >>> r_up2\n    array([[-23.79075451, -41.1084062 ,  24.71548348],  # may vary (signs)\n           [  0.        , -33.83931057,  11.02226551],\n           [  0.        ,   0.        ,  48.91476811],\n           [  0.        ,   0.        ,   0.        ],\n           [  0.        ,   0.        ,   0.        ]])\n\n    This update is also a valid qr decomposition of ``A + U V**T``.\n\n    >>> a_up2 = a + np.dot(u2, v2.T)\n    >>> np.allclose(a_up2, np.dot(q_up2, r_up2))\n    True\n    >>> np.allclose(np.dot(q_up2.T, q_up2), np.eye(5))\n    True\n\n    "
    ...

def __getattr__(name) -> typing.Any:
    ...




from numpy.typing import NDArray

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2014, Nicolas P. Rougier
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import numpy as np

def compact(vertices, indices, tolerance=1e-3): ...
def normals(vertices: NDArray, indices: NDArray): ...



from numpy.typing import ArrayLike
from ..classes.graph import Graph
from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["bethe_hessian_matrix"]

def bethe_hessian_matrix(
    G: Graph, r: float | None = None, nodelist: ArrayLike | None = None
): ...



from typing import Any, ClassVar, TypeVar
from numpy.random import RandomState
from numpy import ndarray
from ..utils._param_validation import Interval as Interval
from numbers import Real as Real
from ..base import OutlierMixin
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..metrics import accuracy_score as accuracy_score
from ..utils.validation import check_is_fitted as check_is_fitted
from . import MinCovDet

EllipticEnvelope_Self = TypeVar("EllipticEnvelope_Self", bound="EllipticEnvelope")

# Author: Virgile Fritsch <virgile.fritsch@inria.fr>
#
# License: BSD 3 clause

import numpy as np


class EllipticEnvelope(OutlierMixin, MinCovDet):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dist_: ndarray = ...
    raw_support_: ndarray = ...
    raw_covariance_: ndarray = ...
    raw_location_: ndarray = ...
    offset_: float = ...
    support_: ndarray = ...
    precision_: ndarray = ...
    covariance_: ndarray = ...
    location_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        store_precision: bool = True,
        assume_centered: bool = False,
        support_fraction: None | Float = None,
        contamination: Float = 0.1,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: EllipticEnvelope_Self, X: MatrixLike, y: Any = None
    ) -> EllipticEnvelope_Self:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def score_samples(self, X: MatrixLike) -> ArrayLike:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def score(
        self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> float:
        ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.io._test_fortran, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def read_unformatted_double(m, n, k, filename) -> typing.Any:
    "a = read_unformatted_double(m,n,k,filename)\n\nWrapper for ``read_unformatted_double``.\n\nParameters\n----------\nm : input int\nn : input int\nk : input int\nfilename : input string(len=4096)\n\nReturns\n-------\na : rank-3 array('d') with bounds (m,n,k)\n"
    ...

def read_unformatted_int(m, n, k, filename) -> typing.Any:
    "a = read_unformatted_int(m,n,k,filename)\n\nWrapper for ``read_unformatted_int``.\n\nParameters\n----------\nm : input int\nn : input int\nk : input int\nfilename : input string(len=4096)\n\nReturns\n-------\na : rank-3 array('i') with bounds (m,n,k)\n"
    ...

def read_unformatted_mixed(m, n, k, filename) -> typing.Any:
    "a,b = read_unformatted_mixed(m,n,k,filename)\n\nWrapper for ``read_unformatted_mixed``.\n\nParameters\n----------\nm : input int\nn : input int\nk : input int\nfilename : input string(len=4096)\n\nReturns\n-------\na : rank-2 array('d') with bounds (m,n)\nb : rank-1 array('i') with bounds (k)\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from ._find_contours import find_contours as find_contours
from ._marching_cubes_lewiner import marching_cubes as marching_cubes
from ._marching_cubes_classic import mesh_surface_area as mesh_surface_area
from ._regionprops import (
    regionprops as regionprops,
    perimeter as perimeter,
    perimeter_crofton as perimeter_crofton,
    euler_number as euler_number,
    regionprops_table as regionprops_table,
)
from ._polygon import (
    approximate_polygon as approximate_polygon,
    subdivide_polygon as subdivide_polygon,
)
from .pnpoly import (
    points_in_poly as points_in_poly,
    grid_points_in_poly as grid_points_in_poly,
)
from ._moments import (
    moments as moments,
    moments_central as moments_central,
    moments_coords as moments_coords,
    moments_coords_central as moments_coords_central,
    moments_normalized as moments_normalized,
    centroid as centroid,
    moments_hu as moments_hu,
    inertia_tensor as inertia_tensor,
    inertia_tensor_eigvals as inertia_tensor_eigvals,
)
from .profile import profile_line as profile_line
from .fit import (
    LineModelND as LineModelND,
    CircleModel as CircleModel,
    EllipseModel as EllipseModel,
    ransac as ransac,
)
from .block import block_reduce as block_reduce
from ._label import label as label
from .entropy import shannon_entropy as shannon_entropy
from ._blur_effect import blur_effect as blur_effect

__all__ = [
    "find_contours",
    "regionprops",
    "regionprops_table",
    "perimeter",
    "perimeter_crofton",
    "euler_number",
    "approximate_polygon",
    "subdivide_polygon",
    "LineModelND",
    "CircleModel",
    "EllipseModel",
    "ransac",
    "block_reduce",
    "moments",
    "moments_central",
    "moments_coords",
    "moments_coords_central",
    "moments_normalized",
    "moments_hu",
    "inertia_tensor",
    "inertia_tensor_eigvals",
    "marching_cubes",
    "mesh_surface_area",
    "profile_line",
    "label",
    "points_in_poly",
    "grid_points_in_poly",
    "shannon_entropy",
    "blur_effect",
]



from typing import Mapping, Any
from ..util.event import Event


# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from copy import deepcopy
import weakref

from .glir import GlirQueue, BaseGlirParser, GlirParser, glir_logger
from .wrappers import BaseGlooFunctions
from .. import config

_default_dict = ...

canvasses: list = ...

def get_default_config() -> Mapping: ...
def get_current_canvas(): ...
def set_current_canvas(canvas): ...
def forget_canvas(canvas): ...

class GLContext(BaseGlooFunctions):
    def __init__(self, config: Mapping | None = None, shared: None | GLContext = None): ...
    def __repr__(self): ...
    def _set_config(self, config): ...
    def create_shared(self, name: str, ref: Any): ...
    @property
    def config(self): ...
    @property
    def glir(self): ...
    @property
    def shared(self): ...
    @property
    def capabilities(self): ...
    def flush_commands(self, event: Event | None = None): ...
    def set_viewport(self, *args): ...
    def get_viewport(self): ...

class GLShared(object):

    # We keep a (weak) ref of each backend that gets associated with
    # this object. In theory, this means that multiple canvases can
    # be created and also deleted; as long as there is at least one
    # left, things should Just Work.

    def __init__(self): ...
    def __repr__(self): ...
    @property
    def parser(self): ...
    @parser.setter
    def parser(self, parser): ...
    def add_ref(self, name, ref): ...
    @property
    def name(self): ...
    @property
    def ref(self): ...

class FakeCanvas(object):
    def __init__(self): ...
    def flush(self): ...
